package org.alfresco.bm.publicapi;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Random;
import java.util.UUID;

import org.alfresco.bm.Services;
import org.alfresco.bm.event.selector.AbstractEventSelectorProcessor;
import org.alfresco.bm.event.selector.EventSelector;
import org.alfresco.bm.publicapi.data.ContentCreator;
import org.alfresco.bm.publicapi.data.DefaultContentCreator;
import org.alfresco.bm.publicapi.data.DocumentNode;
import org.alfresco.bm.publicapi.data.FolderNode;
import org.alfresco.bm.publicapi.data.Node;
import org.alfresco.bm.publicapi.data.NodesContainer;
import org.alfresco.bm.publicapi.data.Request;
import org.alfresco.bm.publicapi.data.nodematchers.CanCheckOutNodeMatcher;
import org.alfresco.bm.publicapi.data.nodematchers.CanCheckinNodeMatcher;
import org.alfresco.bm.publicapi.data.nodematchers.CanCreateDocumentNodeMatcher;
import org.alfresco.bm.publicapi.data.nodematchers.CanCreateFolderNodeMatcher;
import org.alfresco.bm.publicapi.data.nodematchers.CanDeleteContentNodeMatcher;
import org.alfresco.bm.publicapi.data.nodematchers.CanDeleteNodeMatcher;
import org.alfresco.bm.publicapi.data.nodematchers.CanGetAllVersionsNodeMatcher;
import org.alfresco.bm.publicapi.data.nodematchers.CanGetChildrenNodeMatcher;
import org.alfresco.bm.publicapi.data.nodematchers.CanGetDescendantsNodeMatcher;
import org.alfresco.bm.publicapi.data.nodematchers.CanGetPropertiesNodeMatcher;
import org.alfresco.bm.publicapi.data.nodematchers.CanUpdateContentNodeMatcher;
import org.alfresco.bm.publicapi.data.nodematchers.CanUpdatePropertiesNodeMatcher;
import org.alfresco.bm.publicapi.data.nodematchers.CurrentNodeMatcher;
import org.alfresco.bm.publicapi.data.nodematchers.LockCheckerNodeMatcher;
import org.alfresco.bm.publicapi.data.nodematchers.MatchAllNodesNodeMatcher;
import org.alfresco.bm.publicapi.factory.PublicApiFactory;
import org.alfresco.bm.site.SiteData;
import org.alfresco.bm.site.SiteMember;
import org.alfresco.bm.user.UserData;
import org.apache.chemistry.opencmis.client.api.CmisObject;
import org.apache.chemistry.opencmis.client.api.Session;
import org.apache.chemistry.opencmis.commons.PropertyIds;
import org.springframework.social.alfresco.api.Alfresco;
import org.springframework.social.alfresco.api.entities.Activity.Who;
import org.springframework.social.alfresco.api.entities.AlfrescoList;
import org.springframework.social.alfresco.api.entities.Comment;
import org.springframework.social.alfresco.api.entities.Container;
import org.springframework.social.alfresco.api.entities.Document;
import org.springframework.social.alfresco.api.entities.DocumentFavouriteTarget;
import org.springframework.social.alfresco.api.entities.Favourite;
import org.springframework.social.alfresco.api.entities.FavouriteTarget;
import org.springframework.social.alfresco.api.entities.Folder;
import org.springframework.social.alfresco.api.entities.FolderFavouriteTarget;
import org.springframework.social.alfresco.api.entities.Role;
import org.springframework.social.alfresco.api.entities.Site;
import org.springframework.social.alfresco.api.entities.SiteFavouriteTarget;

/**
 * Event processor that uses an eventSelector to choose the next event and provides functionality
 * useful to public api benchmark tests.
 * 
 * @author steveglover
 */
public abstract class AbstractRepositoryEventSelectorProcessor extends AbstractEventSelectorProcessor
{
    public static final String DEFAULT_ADMIN_USER = "admin";
    public static final String DEFAULT_ADMIN_PWD = "admin";

    protected Services services;
    protected NodeFinder nodeFinder;
    private PublicApiFactory publicApiFactory;
    private ContentCreator contentCreator = new DefaultContentCreator();
    
    protected Random random = new Random(System.currentTimeMillis());

    private PathGenerator pathGenerator = new PathGenerator();

    protected MatchAllNodesNodeMatcher matchAllNodes = new MatchAllNodesNodeMatcher();
    protected CanCreateFolderNodeMatcher canCreateFolderMatcher = new CanCreateFolderNodeMatcher();
    protected CanCreateDocumentNodeMatcher canCreateDocumentMatcher = new CanCreateDocumentNodeMatcher();
    protected CanDeleteNodeMatcher canDeleteNodeMatcher = new CanDeleteNodeMatcher();
    protected CanGetChildrenNodeMatcher canGetChildrenMatcher = new CanGetChildrenNodeMatcher();
    protected CanGetDescendantsNodeMatcher canGetDescendantsMatcher = new CanGetDescendantsNodeMatcher();
    protected CanDeleteContentNodeMatcher canDeleteContentMatcher = new CanDeleteContentNodeMatcher();
    protected LockCheckerNodeMatcher lockCheckerMatcher = new LockCheckerNodeMatcher();
    protected CanUpdatePropertiesNodeMatcher canUpdatePropertiesNodeMatcher = new CanUpdatePropertiesNodeMatcher();
    protected CanGetAllVersionsNodeMatcher canGetAllVersionsNodeMatcher = new CanGetAllVersionsNodeMatcher();
    protected CanCheckOutNodeMatcher canCheckOutNodeMatcher = new CanCheckOutNodeMatcher();
    protected CanCheckinNodeMatcher canCheckinNodeMatcher = new CanCheckinNodeMatcher();
    protected CanUpdateContentNodeMatcher canUpdateContentNodeMatcher = new CanUpdateContentNodeMatcher();
    protected CanGetPropertiesNodeMatcher canGetPropertiesNodeMatcher = new CanGetPropertiesNodeMatcher();
    protected CurrentNodeMatcher currentNodeMatcher = new CurrentNodeMatcher();
    
    public AbstractRepositoryEventSelectorProcessor(Services services, PublicApiFactory publicApiFactory, EventSelector eventSelector)
    {
        super(eventSelector, services.getSessionService());
        this.services = services;
        this.publicApiFactory = publicApiFactory;
        this.nodeFinder = new NodeFinder();
    }

    protected Alfresco getPublicApi(String userId) throws Exception
    {
        Alfresco alfresco = publicApiFactory.getPublicApi(userId);
        return alfresco;
    }

    protected Alfresco getTenantAdminPublicApi(String networkId) throws Exception
    {
        Alfresco alfresco = publicApiFactory.getTenantAdminPublicApi(networkId);
        return alfresco;
    }

    protected Alfresco getAdminPublicApi() throws Exception
    {
        Alfresco alfresco = publicApiFactory.getAdminPublicApi();
        return alfresco;
    }

    public interface NodeMatcher
    {
        boolean matches(Request request, Node input);
    }

    protected String randomSearchString()
    {
    	String searchString = contentCreator.randomSearchString();
    	return searchString;
    }
    
    private String homeSiteName(String userId)
    {
    	return userId.replaceAll(".", "-").replaceAll("@", "-");
    }

    @SuppressWarnings("rawtypes")
    protected Site findSite(Request request, Object data, String expectedCreatedBy) throws Exception
    {
        Site site = null;

        if(data == null)
        {
            return null;
        }
        else if(data instanceof Site)
        {
        	if(expectedCreatedBy != null)
        	{
	            Site foundSite = (Site)data;
	            SiteData siteData = services.getSiteDataService().findSiteBySiteId(foundSite.getId());
	            // note: there will be no site data for home sites
	            if(siteData != null)
	            {
    	            String createdBy = siteData.getCreatedBy();
    	            if(expectedCreatedBy.equals(createdBy))
    	            {
    	            	site = foundSite;
    	            }
	            }
	            else if(foundSite.getId().equals(homeSiteName(expectedCreatedBy)))
	            {
	            	// the site is the user's home site
	            	site = foundSite;
	            }
        	}
        	else
        	{
	            site = (Site)data;
        	}
        }
        else if(data instanceof AlfrescoList)
        {
            AlfrescoList alfrescoList = (AlfrescoList)data;
            ArrayList<?> list = alfrescoList.getEntries();
            return findSite(request, list, expectedCreatedBy);
        }
        else if(data instanceof List)
        {
        	List<?> list = (List<?>)data;
            int size = list.size();

            if(size > 0)
            {
                int index = random.nextInt(size);
                int count = 0;

                while(true)
                {
                	Object obj = list.get(index);
                    if(obj instanceof Site)
                    {
                    	if(expectedCreatedBy != null)
                    	{
            	            Site foundSite = (Site)obj;
            	            SiteData siteData = services.getSiteDataService().findSiteBySiteId(foundSite.getId());
            	            // note: there will be no site data for home sites
            	            if(siteData != null)
            	            {
	            	            String createdBy = siteData.getCreatedBy();
	            	            if(expectedCreatedBy.equals(createdBy))
	            	            {
	            	            	site = foundSite;
	            	            	break;
	            	            }
            	            }
            	            else if(foundSite.getId().equals(homeSiteName(expectedCreatedBy)))
            	            {
            	            	// the site is the user's home site
            	            	site = foundSite;
            	            }
                    	}
                    	else
                    	{
            	            site = (Site)obj;
            	            break;
                    	}
                    }
                    index++;
                    count++;
                    if(count > size)
                    {
                        break;
                    }
                    if(index >= size)
                    {
                        index = 0;
                    }
                }
            }
        }
        else
        {
            throw new IllegalArgumentException("Couldn't get site from input " + data + ", previous request is " + request);
        }

        return site;
    }
    
    protected Site findSite(Request request, Object data) throws Exception
    {
        return findSite(request, data, null);
    }
    
    @SuppressWarnings("rawtypes")
    protected SiteMember findSiteMember(Request request, Object data) throws Exception
    {
        SiteMember siteMember = null;

        if(data == null)
        {
            return null;
        }
        else if(data instanceof SiteMember)
        {
        	siteMember = (SiteMember)data;
        }
        else if(data instanceof AlfrescoList)
        {
            AlfrescoList alfrescoList = (AlfrescoList)data;
            ArrayList<?> list = alfrescoList.getEntries();
            return findSiteMember(request, list);
        }
        else if(data instanceof List)
        {
        	List<?> list = (List<?>)data;
            int size = list.size();

            if(size > 0)
            {
                int index = random.nextInt(size);
                int count = 0;

                while(true)
                {
                	Object obj = list.get(index);
                    if(obj instanceof SiteMember)
                    {
                    	siteMember = (SiteMember)obj;
                        break;
                    }
                    index++;
                    count++;
                    if(count > size)
                    {
                        break;
                    }
                    if(index >= size)
                    {
                        index = 0;
                    }
                }
            }
        }
        else
        {
            throw new IllegalArgumentException("Couldn't get site member from input " + data + ", previous request is " + request);
        }

        return siteMember;
    }
    
    @SuppressWarnings({ "rawtypes" })
    protected DocumentNode getDocument(Request request, Object input, NodeMatcher nodeMatcher) throws Exception
    {
        DocumentNode node = null;

        if(input == null)
        {
            return null;
        }
        else if(input instanceof FolderNode)
        {
            FolderNode folder = (FolderNode)input;
            // randomly select a folder node
            node = nodeFinder.selectRandomDocumentNode(request, folder, nodeMatcher);
        }
        else if(input instanceof DocumentNode)
        {
            DocumentNode docNode = (DocumentNode)input;
            if(nodeMatcher.matches(request, docNode))
            {
                return docNode;        
            }
            else
            {
                return null;
            }
        }
        else if(input instanceof AlfrescoList)
        {
            AlfrescoList alfrescoList = (AlfrescoList)input;
            return nodeFinder.findDocumentNode(request, alfrescoList, nodeMatcher);
        }
        else if(input instanceof List)
        {
            List list = (List)input;
            return nodeFinder.findDocumentNode(request, list, nodeMatcher);
        }
        else if(input instanceof NodesContainer)
        {
            NodesContainer container = (NodesContainer)input;
            if(container.getDocumentNodes().size() > 0)
            {
                node = nodeFinder.findDocumentNode(request, container, nodeMatcher);
            }
        }
        else if(input instanceof Comment)
        {
        	// we have a comment, and can get the node id against which the comment was created
        	String runAsUserId = request.getRunAsUserId();
        	String networkId = request.getNetworkId();
        	
        	Comment comment = (Comment)input;
        	String nodeId = comment.getNodeId();
        	Session cmisSession = getPublicApi(runAsUserId).getCMISSession(networkId);
        	CmisObject cmisObject = cmisSession.getObject(nodeId);
        	if(cmisObject != null)
        	{
        		Node commentedNode = Node.createNode(cmisObject);
        		if(commentedNode instanceof DocumentNode && nodeMatcher.matches(request, commentedNode))
        		{
        			node = (DocumentNode)commentedNode;
        		}
        	}
        }
        else
        {
            throw new IllegalArgumentException("Cannot get node for input " + input + " of type " + input.getClass().getName());
        }
        
        return node;
    }

    public Node findNode(Request request, NodesContainer nodesContainer, NodeMatcher nodeMatcher) throws Exception
    {
        Node ret = null;
    
        List<FolderNode> folderNodes = nodesContainer.getFolderNodes();
        List<DocumentNode> documentNodes = nodesContainer.getDocumentNodes();

        int numFolders = folderNodes.size();
        int numDocs = documentNodes.size();
        int total = numFolders + numDocs;

        if(total > 0)
        {
            List<Node> combined = new ArrayList<Node>(total);
            combined.addAll(folderNodes);
            combined.addAll(documentNodes);
            
            ret = nodeFinder.findNode(request, combined, nodeMatcher);
        }

        return ret;
    }
    
    protected DocumentNode getDocument(Request request, Object input) throws Exception
    {
        return getDocument(request, input, matchAllNodes);
    }
    
    protected String getPath(Request request, Object input) throws Exception
    {
    	String path = pathGenerator.getPath(request, input, matchAllNodes);
    	return path;
    }

    protected Node getNode(Request request, Object input, NodeMatcher nodeMatcher) throws Exception
    {
    	return nodeFinder.findNode(request, input, nodeMatcher);
    }
    
    protected FolderNode getFolder(Request request, Object data, NodeMatcher nodeMatcher) throws Exception
    {
    	return nodeFinder.findFolderNode(request, data, nodeMatcher);
    }

    @SuppressWarnings("rawtypes")
    protected Favourite getFavourite(Request request, Object data)
    {
    	Favourite favourite = null;

        if(data == null)
        {
            return null;
        }
        else if(data instanceof AlfrescoList)
        {
            AlfrescoList alfrescoList = (AlfrescoList)data;
            ArrayList<?> list = alfrescoList.getEntries();
            favourite = getFavourite(request, list);
        }
        else if(data instanceof List)
        {
            List list = (List)data;
            for(Object obj : list)
            {
            	if(obj instanceof Favourite)
            	{
            		favourite = (Favourite)obj;
            		break;
            	}
            }
        }
        else
        {
            throw new IllegalArgumentException("Couldn't get favourite from input " + data + ", previous request is " + request);
        }

        return favourite;
    }
    
    /*
     * Generate a favourite from the input data. Assumes that the data is either sites, document or folders,
     * if not there is no randomness in this method to ensure that site, document and folders favourites
     * are chosen equally. 
     */
    protected Favourite createFavourite(Request request, Object data) throws Exception
    {
	    FavouriteTarget target = null;

    	Site site = getSite(request, data);
    	if(site != null)
    	{
    		target = new SiteFavouriteTarget(site);
    	}
    	else
    	{
	    	DocumentNode documentNode = getDocument(request, data, matchAllNodes);
	    	if(documentNode != null)
	    	{
	    		Document document = documentNode.getDocument();
	    		target = new DocumentFavouriteTarget(document);
	    	}
	    	else
	    	{
		    	FolderNode folderNode = getFolder(request, data, matchAllNodes);
		    	if(folderNode != null)
		    	{
		    		Folder folder = folderNode.getFolder();
		    		target = new FolderFavouriteTarget(folder);
		    	}
	    	}
    	}

	    Favourite favourite = new Favourite();
	    favourite.setTarget(target);

	    return favourite;
    }
    
    @SuppressWarnings("rawtypes")
    protected Site getSite(Request request, Object data)
    {
        Site site = null;

        if(data == null)
        {
            return null;
        }
        else if(data instanceof AlfrescoList)
        {
            AlfrescoList alfrescoList = (AlfrescoList)data;
            ArrayList<?> list = alfrescoList.getEntries();
            for(Object obj : list)
            {
            	if(obj instanceof Site)
            	{
            		site = (Site)obj;
            		break;
            	}
            }
        }
        else if(data instanceof List)
        {
            List list = (List)data;
            for(Object obj : list)
            {
            	if(obj instanceof Site)
            	{
            		site = (Site)obj;
            		break;
            	}
            }
        }
        else
        {
            throw new IllegalArgumentException("Couldn't get site from input " + data + ", previous request is " + request);
        }

        return site;
    }

    /**
     * @return          a random network or <tt>null</tt> if none exists
     */
    protected String randomNetwork()
    {
        UserData user = services.getUserDataService().randomUser();
        return user == null ? null : user.getDomain();
    }

    protected SiteMember randomSiteMember(String userId, String networkId, boolean created)
    {
        SiteMember SiteMember = services.getSiteDataService().randomSiteMemberInNetwork(userId, networkId, created);
        return SiteMember;
    }
    
    protected String randomSite(String networkId)
    {
        SiteData site = services.getSiteDataService().randomSite(networkId);
        return (site != null ? site.getSiteId() : null);
    }
    
    protected String randomSiteForMember(String memberId, String networkId)
    {
        SiteData site = services.getSiteDataService().randomSiteForMember(memberId, networkId);
        return (site != null ? site.getSiteId() : null);
    }
    
    protected String randomSiteForCreator(String creatorId)
    {
        SiteData site = services.getSiteDataService().randomSiteForCreator(creatorId, true);
        return (site != null ? site.getSiteId() : null);
    }

    protected SiteMember randomMember(String siteId)
    {
        SiteMember member = services.getSiteDataService().randomMember(siteId);
        return member;
    }

    protected Role randomRole()
    {
        Role[] roles = Role.values();
        int idx = random.nextInt(roles.length);
        return roles[idx];
    }
    
    protected Who randomWho()
    {
        Who[] values = Who.values();
        int idx = random.nextInt(values.length);
        return values[idx];
    }

    // TODO a selection of properties to update
    protected Map<String, Object> getProperties()
    {
        Map<String, Object> properties = new HashMap<String, Object>();
        properties.put(PropertyIds.NAME, UUID.randomUUID().toString());
        return properties;
    }

    /**
     * A generic map of parameters
     */
    public static class Parameters
    {
        private Map<String, Object> parameters = new HashMap<String, Object>();
        
        public Parameters addParameter(String name, Object value)
        {
        	if(name != null && value != null)
        	{
        		parameters.put(name, value);
        	}
            return this;
        }
        
        public Map<String, String> toMap()
        {
        	Map<String, String> ret = new HashMap<String, String>();
        	for(String key : parameters.keySet())
        	{
        		ret.put(key, String.valueOf(parameters.get(key)));
        	}
            return ret;
        }
    }
    
    /**
     * Utility class to find nodes matching a specific pattern given some input nodes.
     * 
     * @author steveglover
     *
     */
    public class NodeFinder
    {
        private Random random = new Random();
        
        private NodeFinder()
    	{
    		super();
    	}

        public DocumentNode findDocumentNode(Request request, List<?> list, NodeMatcher nodeMatcher)
        {
        	DocumentNode ret = null;
            int size = list.size();

            if(size > 0)
            {
                int index = random.nextInt(size);
                int count = 0;

                while(true)
                {
                    Object obj = list.get(index);
                    if(obj instanceof DocumentNode && nodeMatcher.matches(request, (DocumentNode)obj))
                    {
                    	ret = (DocumentNode)obj;
                        break;
                    }
                    index++;
                    count++;
                    if(count > size)
                    {
                        break;
                    }
                    if(index >= size)
                    {
                        index = 0;
                    }
                }
            }

            return ret;
        }
        
        public DocumentNode findDocumentNode(Request request, AlfrescoList<?> nodes, NodeMatcher nodeMatcher)
        {
            ArrayList<?> list = nodes.getEntries();
            return findDocumentNode(request, list, nodeMatcher);
        }
        
        public FolderNode findFolderNode(Request request, AlfrescoList<?> alfrescoList, NodeMatcher nodeMatcher) throws Exception
        {
            ArrayList<?> list = alfrescoList.getEntries();
            return findFolderNode(request, list, nodeMatcher);
        }
        
        public FolderNode selectRandomFolderNode(Request request, FolderNode fn, NodeMatcher nodeMatcher) throws Exception
        {
            List<FolderNode> folderNodesIncludingThis = new ArrayList<FolderNode>();
            folderNodesIncludingThis.addAll(fn.getFolderNodes());
            folderNodesIncludingThis.add(fn);
            return findFolderNode(request, folderNodesIncludingThis, nodeMatcher);
        }
        
        public DocumentNode selectRandomDocumentNode(Request request, FolderNode fn, NodeMatcher nodeMatcher)
        {
        	List<DocumentNode> documentNodes = fn.getDocumentNodes();
            if(documentNodes.size() > 0)
            {
                return findDocumentNode(request, documentNodes, nodeMatcher);
            }
            else
            {
                return null;
            }
        }

        public FolderNode findFolderNode(Request request, NodesContainer nodesContainer, NodeMatcher nodeMatcher) throws Exception
        {
            List<FolderNode> folderNodes = nodesContainer.getFolderNodes();
            
            if(folderNodes.size() > 0)
            {
                return nodeFinder.findFolderNode(request, folderNodes, nodeMatcher);
            }
            else
            {
                return null;
            }
        }
        
        public DocumentNode findDocumentNode(Request request, NodesContainer nodesContainer, NodeMatcher nodeMatcher)
        {
            List<DocumentNode> documentNodes = nodesContainer.getDocumentNodes();
            if(documentNodes.size() > 0)
            {
                return nodeFinder.findDocumentNode(request, documentNodes, nodeMatcher);
            }
            else
            {
                return null;
            }
        }

        @SuppressWarnings("rawtypes")
        public FolderNode findFolderNode(Request request, Object data, NodeMatcher nodeMatcher) throws Exception
        {
            FolderNode folder = null;

            if(data == null)
            {
                return null;
            }
            else if(data instanceof FolderNode)
            {
                FolderNode f = (FolderNode)data;
                // randomly select a folder node
                folder = selectRandomFolderNode(request, f, nodeMatcher);
            }
            else if(data instanceof Site)
            {
            	Site site = (Site)data;
            	AlfrescoList<Container> containersList = site.getContainers();
            	if(containersList != null)
            	{
            		ArrayList<Container> containers = containersList.getEntries();
            		int size = containers.size();
            		if(size > 0)
            		{
                        int index = random.nextInt(size);
                        int count = 0;

                        while(true)
                        {
                            Container container = containers.get(index);
                            String folderId = container.getId();
        		            FolderNode folderNode = new FolderNode();
        		            folderNode.setNodeId(folderId);
                            if(nodeMatcher.matches(request, folderNode))
                            {
            		            folder = folderNode;
                                break;
                            }
                            index++;
                            count++;
                            if(count > size)
                            {
                                break;
                            }
                            if(index >= size)
                            {
                                index = 0;
                            }
                        }
            		}
            	}
            }
            else if(data instanceof AlfrescoList)
            {
                AlfrescoList alfrescoList = (AlfrescoList)data;
                return findFolderNode(request, alfrescoList, nodeMatcher);
            }
            else if(data instanceof List)
            {
                List list = (List)data;
                return findFolderNode(request, list, nodeMatcher);
            }
            else if(data instanceof NodesContainer)
            {
                NodesContainer container = (NodesContainer)data;
                FolderNode fn = findFolderNode(request, container, nodeMatcher);
                if(fn == null)
                {
                    // no folders in results, randomly select a document and use its parent
                    Node doc = findDocumentNode(request, container, nodeMatcher);
                    if(doc != null)
                    {
                        String folderId = (String)doc.getProperty(PropertyIds.PARENT_ID);
                        if(folderId == null)
                        {
                        	List<FolderNode> parents = doc.getParents();
                        	if(parents != null && parents.size() > 0)
                        	{
                        		folder = parents.get(0); // index 0 is primary parent
                            	if(folder == null)
                            	{
                            		if(parents.size() > 1)
                            		{
    	                        		int index = 1 + random.nextInt(parents.size() - 1);
    	                        		folder = parents.get(index);
                            		}
                            	}
                        	}
                        }
                        else
                        {
                        	// TODO the folder won't have allowableActions, making it difficult to choose successor events
                        	folder = new FolderNode();
                        	folder.setNodeId(folderId);
                        }
                    }
                }
                else
                {
                    folder = fn;
                }
            }
            else if(data instanceof DocumentNode)
            {
                DocumentNode doc = (DocumentNode)data;
                String folderId = (String)doc.getProperty(PropertyIds.PARENT_ID);
                if(folderId == null)
                {
                	List<FolderNode> parents = doc.getParents();
                	if(parents != null && parents.size() > 0)
                	{
                		folder = parents.get(0); // index 0 is primary parent
                    	if(folder == null)
                    	{
                    		if(parents.size() > 1)
                    		{
                        		int index = 1 + random.nextInt(parents.size() - 1);
                        		folder = parents.get(index);
                    		}
                    	}
                	}
                }
                else
                {
                	// TODO the folder won't have allowableActions, making it difficult to choose successor events
                	folder = new FolderNode();
                	folder.setNodeId(folderId);
                }
            }
            else if(data instanceof Comment)
            {
            	// we have a comment, and can get the node id against which the comment was created
            	String runAsUserId = request.getRunAsUserId();
            	String networkId = request.getNetworkId();
            	
            	Comment comment = (Comment)data;
            	String nodeId = comment.getNodeId();
            	Session cmisSession = getPublicApi(runAsUserId).getCMISSession(networkId);
            	CmisObject cmisObject = cmisSession.getObject(nodeId);
            	if(cmisObject != null)
            	{
            		Node commentedNode = Node.createNode(cmisObject);
            		if(commentedNode instanceof FolderNode && nodeMatcher.matches(request, commentedNode))
            		{
            			folder = (FolderNode)commentedNode;
            		}
            	}
            }
            else
            {
                throw new IllegalArgumentException("Couldn't get folder from input " + data + ", previous request is " + request);
            }

            if(folder != null && folder.getNodeId() == null)
            {
                throw new RuntimeException("Node id is null for " + folder);
            }

            return folder;
        }
        
        public FolderNode findFolderNode(Request request, List<?> list, NodeMatcher nodeMatcher) throws Exception
        {
        	FolderNode ret = null;
            int size = list.size();

            if(size > 0)
            {
                int index = random.nextInt(size);
                int count = 0;

                while(true)
                {
                	Object obj = list.get(index);
                	if(obj instanceof FolderNode && nodeMatcher.matches(request, (FolderNode)obj))
                	{
                		ret = (FolderNode)obj;
                		break;
                	}
                    index++;
                    count++;
                    if(count > size)
                    {
                        break;
                    }
                    if(index >= size)
                    {
                        index = 0;
                    }
                }
            }            

            return ret;
        }

        @SuppressWarnings("rawtypes")
    	public Node findNode(Request request, Object input, NodeMatcher nodeMatcher) throws Exception
        {
            Node node = null;

            if(input == null)
            {
                return null;
            }
            else if(input instanceof Node)
            {
                if(nodeMatcher.matches(request, (Node)input))
                {
                    node = (Node)input;                
                }
            }
            else if(input instanceof AlfrescoList)
            {
                AlfrescoList alfrescoList = (AlfrescoList)input;
                return findNode(request, alfrescoList, nodeMatcher);
            }
            else if(input instanceof List)
            {
                List list = (List)input;
                return findNode(request, list, nodeMatcher);
            }
            else if(input instanceof NodesContainer)
            {
                NodesContainer container = (NodesContainer)input;
                node = AbstractRepositoryEventSelectorProcessor.this.findNode(request, container, nodeMatcher);
            }
            else
            {
                throw new IllegalArgumentException("Cannot get node for input " + input + " of type " + input.getClass().getName());
            }

            return node;
        }

        public Node findNode(Request request, AlfrescoList<?> alfrescoList, NodeMatcher nodeMatcher) throws Exception
        {
            ArrayList<?> list = alfrescoList.getEntries();
            return findNode(request, list, nodeMatcher);
        }
        
        public Node findNode(Request request, List<?> list, NodeMatcher nodeMatcher) throws Exception
        {
            Node ret = null;
            int size = list.size();
            
            if(size > 0)
            {
                int index = random.nextInt(size);
                int count = 0;
        
                while(true)
                {
                    Object obj = list.get(index);
                	if(obj instanceof Node && nodeMatcher.matches(request, (Node)obj))
                	{
                		ret = (Node)obj;
                		break;
                	}
                    index++;
                    count++;
                    if(count > size)
                    {
                        break;
                    }
                    if(index >= size)
                    {
                        index = 0;
                    }
                }
            }

            return ret;
        }
    }
    
    /**
     * Generates an object path from some input: a site (with site containers) or node (or collections of these).
     * 
     * @author steveglover
     *
     */
    public class PathGenerator
    {
    	private Random random = new Random();

        /**
         * 
         * @param request the request that generated the input
         * @param input input data from which to generate a path
         * @param nodeMatcher a node matcher to filter out unwanted nodes, if necessary
         * @return
         */
        @SuppressWarnings("rawtypes")
    	public String getPath(Request request, Object input, NodeMatcher nodeMatcher) throws Exception
        {
        	String path = null;

            if(input == null)
            {
            	path = null;
            }
            else if(input instanceof Site)
            {
            	Site site = (Site)input;
            	AlfrescoList<Container> containers = site.getContainers();
            	if(containers != null)
            	{
    	        	ArrayList<Container> list = containers.getEntries();
    	        	if(containers != null && list.size() > 0)
    	        	{
    	        		int index = random.nextInt(list.size());
    	        		Container container = list.get(index);
    	        		path = "/Sites/" + site.getId() + "/" + container.getFolderId();
    	        	}
            	}
            }
            else if(input instanceof SiteMember)
            {
            	SiteMember siteMember = (SiteMember)input;
            	String siteId = siteMember.getSiteId();
            	if(siteId != null)
            	{
            		path = "/Sites/" + siteId + "/documentLibrary";
            	}
            }
            else if(input instanceof AlfrescoList)
            {
            	AlfrescoList list = (AlfrescoList)input;
            	ArrayList entries = list != null ? list.getEntries() : null;
            	if(entries != null && entries.size() > 0)
            	{
            		int size = entries.size();
                    int index = random.nextInt(size);
                    int count = 0;
            
                    while(true)
                    {
                        Object obj = entries.get(index);
                        if(obj instanceof Site)
                        {
                        	path = getPath(request, obj, nodeMatcher);
                            break;
                        }
                        index++;
                        count++;
                        if(count > size)
                        {
                            break;
                        }
                        if(index >= size)
                        {
                            index = 0;
                        }
                    }
            	}
            }
            else if(input instanceof List)
            {
            	List entries = (List)input;
            	if(entries != null && entries.size() > 0)
            	{
            		int size = entries.size();
                    int index = random.nextInt(size);
                    int count = 0;
            
                    while(true)
                    {
                        Object obj = entries.get(index);
                        if(obj instanceof Site)
                        {
                        	path = getPath(request, obj, nodeMatcher);
                            break;
                        }
                        index++;
                        count++;
                        if(count > size)
                        {
                            break;
                        }
                        if(index >= size)
                        {
                            index = 0;
                        }
                    }
            	}
            }
            else if(input instanceof NodesContainer)
            {
            	NodesContainer container = (NodesContainer)input;
            	Node node = findNode(request, container, nodeMatcher);
            	if(node != null)
            	{
            		path = getPath(request, node, nodeMatcher);
            	}
            }
            else if(input instanceof Node)
            {
            	Node node = (Node)input;
            	path = (String)node.getProperty(PropertyIds.PATH);
            }
            else
            {
                throw new IllegalArgumentException("Cannot get path for input " + input + " of type " + input.getClass().getName());
            }

            return path;
        }
    }
}
