package org.alfresco.bm.publicapi.data;

import java.io.IOException;
import java.io.InputStream;
import java.io.StringWriter;
import java.math.BigInteger;

import org.apache.chemistry.opencmis.commons.data.ContentStream;
import org.apache.chemistry.opencmis.commons.impl.dataobjects.ContentStreamImpl;
import org.apache.commons.io.IOUtils;

/**
 * Represents document/node content.
 * 
 * @author steveglover
 *
 */
public abstract class AbstractDocumentContent implements DocumentContent
{
    private String filename;
    
    public AbstractDocumentContent(String filename)
    {
        this.filename = filename;
    }
    
    /**
     * Get the (byte) length of the content. Default is null, override in subclasses.
     * 
     * @return the length (in bytes) of the document content
     */
    public BigInteger getLength()
    {
    	return null;
    }

    public String getFilename()
	{
		return filename;
	}

	/**
     * Get a CMIS content stream for the content.
     * 
     * @return a CMIS content stream to the document content
     */
    public ContentStream getContentStream()
    {
        InputStream is = getInputStream();
        BigInteger length = getLength();
        ContentStream contentStream = new ContentStreamImpl(filename, length, "text/plain", is);
        return contentStream;
    }
    
    /**
     * Convert CMIS content stream to a DocumentContent object.
     * 
     * @param stream
     * @return a DocumentContent object for the CMIS content stream
     * @throws IOException
     */
    public static AbstractDocumentContent getContent(ContentStream stream) throws IOException
    {
        AbstractDocumentContent documentContent = null;

        if(stream != null)
        {
	        String filename = stream.getFileName();
	        InputStream is = stream.getStream();
	        String mimeType = stream.getMimeType();
	        if(mimeType.equals("text/plain"))
	        {
	            StringWriter writer = new StringWriter();
	            IOUtils.copy(is, writer, "UTF-8");
	            String content = writer.toString();
	            documentContent = new StringDocumentContent(filename, content);
	        }
	        else
	        {
	            BigInteger length = BigInteger.valueOf(stream.getLength());
	            documentContent = new BinaryDocumentContent(filename, is, length);
	        }
        }

        return documentContent;
    }

    @Override
    public String toString()
    {
        return "DocumentContent [filename=" + filename
                + "]";
    }
}
