package org.alfresco.bm.publicapi.data;

import java.io.Serializable;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.Random;

import org.alfresco.bm.publicapi.process.CreateDocumentByParentIdRequest;
import org.alfresco.bm.publicapi.process.CreateDocumentByParentPathRequest;
import org.apache.chemistry.opencmis.commons.enums.VersioningState;

/**
 * Generates create document requests.
 * 
 * @author steveglover
 *
 */
public class CreateDocumentRequestFactory extends AbstractCreateNodeRequestFactory
{
	private Random random = new Random();

    private List<VersioningState> documentVersioningStates = Arrays.asList(VersioningState.CHECKEDOUT, VersioningState.MAJOR, VersioningState.MINOR);
    private List<VersioningState> folderVersioningStates = Arrays.asList(VersioningState.NONE);

	private ContentCreator contentCreator;

	public CreateDocumentRequestFactory(ContentCreator contentCreator)
	{
		this.contentCreator = contentCreator;
	}

    private VersioningState randomVersioningState(String objectType)
    {
        VersioningState ret = null;

        if(objectType.equals("cmis:document"))
        {
            int idx = random.nextInt(documentVersioningStates.size());
            ret = documentVersioningStates.get(idx);
        }
        else if(objectType.equals("cmis:folder"))
        {
            int idx = random.nextInt(folderVersioningStates.size());
            ret = folderVersioningStates.get(idx);
        }
        else
        {
            ret = VersioningState.NONE; // TODO
        }

        return ret;
    }

	/**
	 * Generate some node content.
	 * 
	 * @return the node content
	 */
	public CreateDocumentByParentIdRequest create(String networkId, String runAsUserId, FolderNode parentFolder)
	{
        String type = "cmis:document"; // TODO - different types?
        VersioningState versioningState = randomVersioningState(type);
        DocumentContent content = contentCreator.generateContent();
        Map<String, Serializable> properties = getProperties();

        CreateDocumentByParentIdRequest ret = new CreateDocumentByParentIdRequest(networkId, runAsUserId, parentFolder, versioningState, type, content, properties);
        return ret;
	}
	
	public CreateDocumentByParentPathRequest create(String networkId, String runAsUserId, Path parentPath)
	{
        String type = "cmis:document"; // TODO - different types?
        VersioningState versioningState = randomVersioningState(type);
        DocumentContent content = contentCreator.generateContent();
        Map<String, Serializable> properties = getProperties();

        CreateDocumentByParentPathRequest ret = new CreateDocumentByParentPathRequest(networkId, runAsUserId, parentPath, versioningState, type, content, properties);
        return ret;
	}
}
