package org.alfresco.bm.publicapi.data;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Random;
import java.util.UUID;

public class DefaultContentCreator implements ContentCreator
{
	private static final String[] exampleContentNames = new String[] { "cmis-spec-v1.0.pdf" };
	private static final List<String> strings = Arrays.asList("the", "quick", "brown", "fox", "jumped", "over", "the", "brown", "fence");

	private List<DocumentContent> contents;
	private Random random = new Random();

	public DefaultContentCreator()
	{
		this.contents = new ArrayList<DocumentContent>();

		for(String exampleContentName : exampleContentNames)
		{
			InputStream is = getInputFromClasspath(exampleContentName);
			DocumentContent content = new InputStreamDocumentContent(exampleContentName, is);
			contents.add(content);
		}
	}

	private InputStream getInputFromClasspath(String name)
	{
		InputStream is = getClass().getClassLoader().getResourceAsStream(name);
		return is;
	}

	@Override
	public DocumentContent generateBinaryContent()
	{
		int index = random.nextInt(contents.size());
		DocumentContent content = contents.get(index);
		return content;
	}

	@Override
	public DocumentContent generateStringContent()
	{
		String string = randomString();
		DocumentContent content = new StringDocumentContent(UUID.randomUUID().toString(), string);
		return content;
	}
	
	@Override
	public DocumentContent generateContent()
	{
		int idx = random.nextInt(2);
		DocumentContent content = (idx == 0 ? generateBinaryContent() : generateStringContent());
		return content;
	}
	
	public String randomString()
	{
		StringBuilder sb = new StringBuilder();
		for(int i = 0; i < 10; i++)
		{
			int idx = random.nextInt(strings.size());
			sb.append(strings.get(idx));
			sb.append(" ");
		}
		return sb.toString();
	}

	@Override
	public String randomSearchString()
	{
		int idx = random.nextInt(strings.size());
		String s = strings.get(idx);
		return s;
	}
}
