package org.alfresco.bm.publicapi.data;

import java.util.ArrayList;
import java.util.List;

import org.apache.chemistry.opencmis.client.api.FileableCmisObject;
import org.apache.chemistry.opencmis.client.api.Tree;
import org.springframework.social.alfresco.api.entities.Folder;

/**
 * Represents a folder node i.e. cm:folder.
 * 
 * @author steveglover
 *
 */
public class FolderNode extends Node
{
    /*
     * A list of child folder nodes.
     */
    private List<FolderNode> folderNodes = new ArrayList<FolderNode>();
    
    /*
     * A list of child document nodes.
     */
    private List<DocumentNode> documentNodes = new ArrayList<DocumentNode>();

    public FolderNode()
    {
        super();
    }
    
    /**
     * Set the descendants of this node from the CMIS nodeTree parameter.
     * 
     * @param nodeTree
     */
    public void setDescendants(List<Tree<FileableCmisObject>> nodeTree)
    {
        folderNodes.clear();
        documentNodes.clear();

        for(Tree<FileableCmisObject> tree : nodeTree)
        {
            Node node = Node.createNode(tree);
            if(node instanceof FolderNode)
            {
                folderNodes.add((FolderNode)node);
            }
            else
            {
                documentNodes.add((DocumentNode)node);
            }
        }
    }

    public boolean isFolder()
    {
        return true;
    }
    
    public void addChild(Node node)
    {
        if(node instanceof FolderNode)
        {
            addFolder((FolderNode)node);
        }
        else
        {
            addDocument((DocumentNode)node);
        }
    }

    public void addFolder(FolderNode folder)
    {
        folderNodes.add(folder);
    }
    
    public void addDocument(DocumentNode node)
    {
        documentNodes.add(node);
    }
    
    public List<FolderNode> getFolderNodes()
    {
        return folderNodes;
    }

    public List<DocumentNode> getDocumentNodes()
    {
        return documentNodes;
    }

    public Folder getFolder()
    {
    	Folder folder = new Folder();
    	folder.setId(getNodeId());
    	return folder;
    }
    
    @Override
    public String toString()
    {
        return "FolderNode [folderNodes=" + folderNodes + ", documentNodes="
                + documentNodes + ", nodeId=" + nodeId + ", properties="
                + properties + ", parents=" + parents
                + ", allowableActions=" + allowableActions + ", tags=" + tags
                + ", comments=" + comments + ", ratings=" + ratings + "]";
    }
}
