package org.alfresco.bm.publicapi.data;

import java.util.LinkedList;
import java.util.List;

import org.apache.chemistry.opencmis.client.api.Document;
import org.apache.chemistry.opencmis.client.api.ItemIterable;

/**
 * A container of documents and folders. Useful for being able to randomly select a document or a folder.
 * 
 * @author steveglover
 *
 */
public class NodesContainer
{
    private List<FolderNode> folderNodes = new LinkedList<FolderNode>();
    private List<DocumentNode> documentNodes = new LinkedList<DocumentNode>();

    public NodesContainer()
    {
    }
    
    public NodesContainer(List<Document> docs)
    {
    	for(Document doc : docs)
    	{
    		addDocument((DocumentNode)Node.createNode(doc));
    	}
    }
    
    public NodesContainer(ItemIterable<Document> docs)
    {
    	for(Document doc : docs)
    	{
    		addDocument((DocumentNode)Node.createNode(doc));
    	}
    }

    public void addFolder(FolderNode folder)
    {
        folderNodes.add(folder);
    }

    public void addDocument(DocumentNode doc)
    {
        documentNodes.add(doc);
    }

    public void addNode(Node node)
    {
        if(node instanceof FolderNode)
        {
            addFolder((FolderNode)node);
        }
        else if(node instanceof DocumentNode)
        {
            addDocument((DocumentNode)node);
        }
    }
    
    public List<FolderNode> getFolderNodes()
    {
        return folderNodes;
    }

    public List<DocumentNode> getDocumentNodes()
    {
        return documentNodes;
    }

//    public Node findNode(Request request, NodeMatcher nodeMatcher) throws Exception
//    {
//        Node ret = null;
//    
//        int numFolders = folderNodes.size();
//        int numDocs = documentNodes.size();
//        int total = numFolders + numDocs;
//
//        if(total > 0)
//        {
//            List<Node> combined = new ArrayList<Node>(total);
//            combined.addAll(folderNodes);
//            combined.addAll(documentNodes);
//            
//            ret = nodeFinder.findNode(request, combined, nodeMatcher);
//        }
//
//        return ret;
//    }
//
//    public FolderNode findFolderNode(Request request, NodeMatcher nodeMatcher) throws Exception
//    {
//        if(folderNodes.size() > 0)
//        {
//            return nodeFinder.findFolderNode(request, folderNodes, nodeMatcher);
//        }
//        else
//        {
//            return null;
//        }
//    }
//    
//    public DocumentNode findDocumentNode(Request request, NodeMatcher nodeMatcher)
//    {
//        if(documentNodes.size() > 0)
//        {
//            return nodeFinder.findDocumentNode(request, documentNodes, nodeMatcher);
//        }
//        else
//        {
//            return null;
//        }
//    }
}
