package org.alfresco.bm.publicapi.data;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.math.BigInteger;
import java.util.Arrays;

/**
 * Textual document node content represented as a string.
 * 
 * @author steveglover
 *
 */
public class StringDocumentContent extends AbstractDocumentContent
{
    private String content;
    private byte[] bytes;
    private BigInteger length;
    
    public StringDocumentContent(String filename, String content)
    {
        super(filename);
        this.content = content;
        try
        {
            this.bytes = content.getBytes("UTF-8");
        }
        catch(UnsupportedEncodingException e)
        {
            this.bytes = content.getBytes();
        }
        long len = bytes.length;
        this.length = BigInteger.valueOf(len);
    }

    @Override
    public InputStream getInputStream()
    {
        ByteArrayInputStream is = new ByteArrayInputStream(bytes);
        return is;
    }

    @Override
    public BigInteger getLength()
    {
        return length;
    }

    @Override
    public String toString()
    {
        return "StringDocumentContent [content=" + content + ", bytes="
                + Arrays.toString(bytes) + "]";
    }
}
