package org.alfresco.bm.publicapi.data.nodematchers;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import org.alfresco.bm.publicapi.AbstractRepositoryEventSelectorProcessor.NodeMatcher;
import org.alfresco.bm.publicapi.data.Node;
import org.alfresco.bm.publicapi.data.Request;
import org.apache.chemistry.opencmis.commons.enums.Action;

public class CanUpdateContentNodeMatcher extends ChainingNodeMatcher
{
	public CanUpdateContentNodeMatcher()
	{
		super(CanUpdateContentNodeMatcher.getMatchers());
	}

	private static List<NodeMatcher> getMatchers()
	{
		// CMIS allowable action "canSetContentStream" does not seem to be set properly in some cases,
		// so check for a node lock too
		List<NodeMatcher> matchers = new ArrayList<NodeMatcher>(2);
		matchers.add(new LockCheckerNodeMatcher());
		matchers.add(new CanSetContentStreamNodeMatcher());
		return matchers;
	}
	
	private static class CanSetContentStreamNodeMatcher implements NodeMatcher
	{
	    @Override
	    public boolean matches(Request request, Node input)
	    {
	        Set<Action> allowableActions = input.getAllowableActions();
	        // If no allowableActions, assume true (non-CMIS public api responses don't include allowableActions)
	        return allowableActions == null || allowableActions.contains(Action.CAN_SET_CONTENT_STREAM);
	    }
	}
}
