package org.alfresco.bm.publicapi.data.nodematchers;

import java.util.Set;

import org.alfresco.bm.publicapi.AbstractRepositoryEventSelectorProcessor.NodeMatcher;
import org.alfresco.bm.publicapi.data.Node;
import org.alfresco.bm.publicapi.data.Request;
import org.apache.chemistry.opencmis.commons.PropertyIds;

/**
 * Matches a Node's objectTypeId against a set of required CMIS objectTypeIds and excluded objectTypeIds.
 * 
 * @author steveglover
 *
 */
public class ObjectTypeIdNodeMatcher implements NodeMatcher
{
    private Set<String> requiredCmisTypes;
    private Set<String> excludeCmisTypes;

    public ObjectTypeIdNodeMatcher(Set<String> requiredCmisTypes, Set<String> excludeCmisTypes)
    {
        this.requiredCmisTypes = requiredCmisTypes;
        this.excludeCmisTypes = excludeCmisTypes;
    }

    @Override
    public boolean matches(Request request, Node input)
    {
        boolean matches = true;

        String objectTypeId = (String)input.getProperty(PropertyIds.OBJECT_TYPE_ID);
        if(requiredCmisTypes.size() > 0)
        {
            if(!requiredCmisTypes.contains(objectTypeId))
            {
                matches = false;
            }
        }
        if(excludeCmisTypes.size() > 0)
        {
            if(excludeCmisTypes.contains(objectTypeId))
            {
                matches = false;
            }
        }

        return matches;
    }
}
