package org.alfresco.bm.publicapi.factory;

import org.alfresco.bm.user.UserData;
import org.alfresco.bm.user.UserDataService;
import org.springframework.social.alfresco.api.Alfresco;
import org.springframework.social.alfresco.api.impl.ConnectionDetails;
import org.springframework.social.alfresco.connect.BasicAuthAlfrescoConnectionFactory;
import org.springframework.social.connect.Connection;

/**
 * A public api factory that uses basic authentication to communicate with a repository.
 * 
 * @author steveglover
 *
 */
public class BasicAuthPublicApiFactory implements PublicApiFactory
{
    private String scheme;
    private String host;
    private int port;
    private int maxNumberOfConnections;
    private int connectionTimeoutMs;
    private int socketTimeoutMs;
    private int socketTtlMs;
    private boolean isProduction;
    
    private UserDataService userDataService;
    
    public BasicAuthPublicApiFactory(String scheme, String host, int port, int maxNumberOfConnections, int connectionTimeoutMs, 
    		int socketTimeoutMs, int socketTtlMs, UserDataService userDataService, boolean isProduction)
    {
        super();
        this.scheme = scheme;
        this.host = host;
        this.port = port;
		this.maxNumberOfConnections= maxNumberOfConnections;
		this.connectionTimeoutMs = connectionTimeoutMs;
		this.socketTimeoutMs = socketTimeoutMs;
		this.socketTtlMs= socketTtlMs; 
        this.userDataService = userDataService;
        this.isProduction = isProduction;
    }
    
    public boolean isProduction()
	{
		return isProduction;
	}

	public String getScheme()
	{
		return scheme;
	}

	public String getHost()
    {
        return host;
    }

    public int getPort()
    {
        return port;
    }
    
    @Override
    public Alfresco getPublicApi(String username)
    {
        Alfresco alfresco = null;
        UserData user = userDataService.findUserByEmail(username);
        if(user != null)
        {
        	ConnectionDetails connectionDetails = new ConnectionDetails(scheme, host, port, username, user.getPassword(), isProduction,
        			maxNumberOfConnections, connectionTimeoutMs, socketTimeoutMs, socketTtlMs);
            BasicAuthAlfrescoConnectionFactory basicAuthConnectionFactory = new BasicAuthAlfrescoConnectionFactory(connectionDetails);
            Connection<Alfresco> connection = basicAuthConnectionFactory.createConnection();
            alfresco = connection.getApi();
        }

        return alfresco;
    }
    
    @Override
    public Alfresco getTenantAdminPublicApi(String networkId)
    {
    	ConnectionDetails connectionDetails = new ConnectionDetails(scheme, host, port, "admin@" + networkId, "admin", isProduction,
        		maxNumberOfConnections, connectionTimeoutMs, socketTimeoutMs, socketTtlMs);
        BasicAuthAlfrescoConnectionFactory connectionFactory = new BasicAuthAlfrescoConnectionFactory(connectionDetails);
        Connection<Alfresco> connection = connectionFactory.createConnection();
        Alfresco alfresco = connection.getApi();
        return alfresco;
    }
    
    @Override
    public Alfresco getAdminPublicApi()
    {
    	ConnectionDetails connectionDetails = new ConnectionDetails(scheme, host, port, "admin", "admin", isProduction,
        		maxNumberOfConnections, connectionTimeoutMs, socketTimeoutMs, socketTtlMs);
        BasicAuthAlfrescoConnectionFactory connectionFactory = new BasicAuthAlfrescoConnectionFactory(connectionDetails);
        Connection<Alfresco> connection = connectionFactory.createConnection();
        Alfresco alfresco = connection.getApi();
        return alfresco;
    }
}
