package org.alfresco.bm.publicapi.factory;

import java.util.List;

import org.alfresco.bm.user.UserData;
import org.alfresco.bm.user.UserDataService;
import org.openqa.selenium.WebElement;
import org.openqa.selenium.htmlunit.HtmlUnitDriver;
import org.springframework.social.alfresco.api.Alfresco;
import org.springframework.social.alfresco.connect.AlfrescoConnectionFactory;
import org.springframework.social.connect.Connection;
import org.springframework.social.oauth2.AccessGrant;
import org.springframework.social.oauth2.GrantType;
import org.springframework.social.oauth2.OAuth2Parameters;

/**
 * A public api factory that uses oauth authentication to communicate with a repository.
 * 
 * @author steveglover
 *
 */
public class OAuthPublicApiFactory implements PublicApiFactory
{
    private static final String STATE = "test";

    private String host;
    private int port;
    
    private String redirectUri;
    private String consumerKey;
    private String consumerSecret;
    
    private UserDataService userDataService;
    
    public OAuthPublicApiFactory(String host, int port, UserDataService userDataService, String redirectUri, String consumerKey, String consumerSecret)
    {
        super();
        this.host = host;
        this.port = port;
        this.userDataService = userDataService;
        this.redirectUri = redirectUri;
        this.consumerKey = consumerKey;
        this.consumerSecret = consumerSecret;
    }
    
    public String getHost()
    {
        return host;
    }

    public int getPort()
    {
        return port;
    }

    @Override
    public Alfresco getPublicApi(String username) throws Exception
    {
        Alfresco alfresco = null;
        UserData user = userDataService.findUserByEmail(username);
        if(user != null)
        {
            AlfrescoConnectionFactory oauthConnectionFactory = new AlfrescoConnectionFactory("https://api.alfresco.com/", consumerKey, consumerSecret, true);

            OAuth2Parameters parameters = new OAuth2Parameters();
            parameters.setRedirectUri(redirectUri);
            parameters.setScope(Alfresco.DEFAULT_SCOPE);
            parameters.setState(STATE);

            String authUrl = oauthConnectionFactory.getOAuthOperations().buildAuthenticateUrl(GrantType.AUTHORIZATION_CODE, parameters);
            
            HtmlUnitDriver driver = new HtmlUnitDriver();
            driver.get(authUrl);

            List<WebElement> webElements = driver.findElementsByTagName("form");

            WebElement usernameElement = driver.findElementById("username");
            usernameElement.sendKeys(username);
            WebElement passwordElement = driver.findElementById("password");
            passwordElement.sendKeys(user.getPassword());
            webElements.get(0).submit();

            CodeUrl codeUrl = new CodeUrl(driver.getCurrentUrl());

            AccessGrant accessGrant = oauthConnectionFactory.getOAuthOperations().exchangeForAccess(codeUrl.getQueryMap().get(CodeUrl.CODE), redirectUri, null);

            Connection<Alfresco> connection = oauthConnectionFactory.createConnection(accessGrant);
            alfresco = connection.getApi();
        }

        return alfresco;
    }
    
    @Override
    public Alfresco getTenantAdminPublicApi(String networkId)
    {
        throw new UnsupportedOperationException();
    }
    
    @Override
    public Alfresco getAdminPublicApi()
    {
        throw new UnsupportedOperationException();
    }
}
