package org.alfresco.bm.publicapi.process;

import org.alfresco.bm.Services;
import org.alfresco.bm.event.selector.EventProcessorResponse;
import org.alfresco.bm.event.selector.EventSelector;
import org.alfresco.bm.publicapi.AbstractRepositoryEventSelectorProcessor;
import org.alfresco.bm.publicapi.data.Request;
import org.alfresco.bm.publicapi.factory.PublicApiFactory;
import org.alfresco.bm.user.UserData;
import org.springframework.social.alfresco.api.entities.Site;

public class AddFavouriteSite extends AbstractRepositoryEventSelectorProcessor
{
    /**
     * @param services                access to collection services
     * @param publicApiFactory        a factory for creating connections to the public api
     * @param eventSelector           the event selector to use in selecting the next event
     */
    public AddFavouriteSite(Services services, PublicApiFactory publicApiFactory, EventSelector eventSelector)
    {
        super(services, publicApiFactory, eventSelector);
    }

    @Override
    public Object createDataObject(Object input, Object resp) throws Exception
    {
    	AddFavouriteSiteRequest ret = null;

        if(input != null)
        {
	        Request request = (Request)input;
	        String userId = request.getRunAsUserId();
	        String networkId = request.getNetworkId();
	        UserData person = services.getUserDataService().randomUser();
	        String siteId = randomSite(networkId);
	
	        ret = new AddFavouriteSiteRequest(networkId, userId, person.getUsername(), siteId);
        }
        
        return ret;
    }

    @Override
    protected EventProcessorResponse processEventImpl(Object input) throws Exception
    {
        EventProcessorResponse response = null;

        if(input == null)
        {
        	 response = new EventProcessorResponse("Unable to add favourite site, input is null", true, null);
        }
        else
        {
	        AddFavouriteSiteRequest addFavouriteSiteRequest = (AddFavouriteSiteRequest)input;
	        String userId = addFavouriteSiteRequest.getRunAsUserId();
	        String networkId = addFavouriteSiteRequest.getNetworkId();
	        String personId = addFavouriteSiteRequest.getPersonId();
	        String siteId = addFavouriteSiteRequest.getSiteId();
	
	        if(siteId != null && userId != null && networkId != null)
	        {
	            Site resp = getPublicApi(userId).addFavoriteSite(networkId, personId, siteId);
	            response = new EventProcessorResponse("Added favourite site " + siteId, true, resp, true);
	        }
	        else
	        {
	            response = new EventProcessorResponse("Skipping add favourite site: no siteId supplied.", false, null);
	        }
        }

        return response;
    }
}
