package org.alfresco.bm.publicapi.process;

import java.util.UUID;

import org.alfresco.bm.Services;
import org.alfresco.bm.event.selector.EventProcessorResponse;
import org.alfresco.bm.event.selector.EventSelector;
import org.alfresco.bm.publicapi.AbstractRepositoryEventSelectorProcessor;
import org.alfresco.bm.publicapi.data.Request;
import org.alfresco.bm.publicapi.factory.PublicApiFactory;
import org.springframework.social.alfresco.api.entities.Tag;

public class ChangeTag extends AbstractRepositoryEventSelectorProcessor
{
    /**
     * @param services                access to collection services
     * @param publicApiFactory        a factory for creating connections to the public api
     * @param eventSelector           the event selector to use in selecting the next event
     */
    public ChangeTag(Services services, PublicApiFactory publicApiFactory, EventSelector eventSelector)
    {
        super(services, publicApiFactory, eventSelector);
    }

    @Override
    public Object createDataObject(Object input, Object resp) throws Exception
    {
    	ChangeTagRequest ret = null;

        if(input != null)
        {
	        Request request = (Request)input;
	        String tagId = null;
	        String newTag = UUID.randomUUID().toString();
	        String networkId = request.getNetworkId();
	        String userId = request.getRunAsUserId();
	        ret = new ChangeTagRequest(networkId, userId, tagId, newTag);
        }
        
        return ret;
    }

    @Override
    protected EventProcessorResponse processEventImpl(Object input) throws Exception
    {
        EventProcessorResponse response = null;

        if(input == null)
        {
        	 response = new EventProcessorResponse("Unable to change tag, input is null", true, null);
        }
        else
        {
	        ChangeTagRequest changeTagRequest = (ChangeTagRequest)input;
	        String userId = changeTagRequest.getRunAsUserId();
	        String networkId = changeTagRequest.getNetworkId();
	        String tagId = changeTagRequest.getTagId();
	        String tag = changeTagRequest.getNewTag();
	
	        if(tagId != null && tag != null && networkId != null && userId != null)
	        {
	            getPublicApi(userId).updateTag(networkId, tagId, tag);
	
	            Tag ret = new Tag();
	            ret.setId(tagId);
	            ret.setTag(tag);
	            response = new EventProcessorResponse("Added tag", true, ret, true);
	        }
	        else
	        {
	            response = new EventProcessorResponse("Failed to add tag", false, null);
	        }
        }

        return response;
    }
}
