package org.alfresco.bm.publicapi.process;

import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

import org.alfresco.bm.Services;
import org.alfresco.bm.event.selector.EventProcessorResponse;
import org.alfresco.bm.event.selector.EventSelector;
import org.alfresco.bm.publicapi.AbstractRepositoryEventSelectorProcessor;
import org.alfresco.bm.publicapi.data.AbstractDocumentContent;
import org.alfresco.bm.publicapi.data.DocumentNode;
import org.alfresco.bm.publicapi.data.Request;
import org.alfresco.bm.publicapi.data.StringDocumentContent;
import org.alfresco.bm.publicapi.factory.PublicApiFactory;
import org.apache.chemistry.opencmis.client.api.Document;
import org.apache.chemistry.opencmis.client.api.ObjectId;
import org.apache.chemistry.opencmis.client.api.Session;

/**
 * Checkin an object.
 * 
 * @author steveglover
 *
 */
public class CheckinObject extends AbstractRepositoryEventSelectorProcessor
{
    /**
     * @param services                access to collection services
     * @param publicApiFactory        a factory for creating connections to the public api
     * @param eventSelector           the event selector to use in selecting the next event
     */
    public CheckinObject(Services services, PublicApiFactory publicApiFactory, EventSelector eventSelector)
    {
        super(services, publicApiFactory, eventSelector);
    }
    
    @Override
    public Object createDataObject(Object input, Object resp) throws Exception
    {
        CheckinObjectRequest ret = null;

        if(input != null)
        {
            Request request = (Request)input;

	        DocumentNode node = getDocument(request, resp, canCheckinNodeMatcher);
	        if(node != null)
	        {
	            logger.debug("CheckinObject.createDataObject node = " + node);
	            
	            String networkId = request.getNetworkId();
	            String userId = request.getRunAsUserId();
	            
	            AbstractDocumentContent content = new StringDocumentContent(UUID.randomUUID().toString(), "TEST CHECKIN CONTENT");
	            boolean major = true; // TODO
	            Map<String, ?> properties = new HashMap<String, String>();
	            String checkinComment = UUID.randomUUID().toString();
	            
	            ret = new CheckinObjectRequest(networkId, userId, node, content, major, properties, checkinComment);
	        }
        }
        
        return ret;
    }

    @Override
    protected EventProcessorResponse processEventImpl(Object input) throws Exception
    {
        EventProcessorResponse response = null;

        if(input == null)
        {
        	 response = new EventProcessorResponse("Unable to checkin object, input is null", true, null);
        }
        else
        {
            CheckinObjectRequest checkinObjectRequest = (CheckinObjectRequest)input;
            String userId = checkinObjectRequest.getRunAsUserId();
            String networkId = checkinObjectRequest.getNetworkId();
            DocumentNode node = checkinObjectRequest.getNode();
            boolean major = checkinObjectRequest.isMajor();
            Map<String, ?> properties = checkinObjectRequest.getProperties();
            String checkinComment = checkinObjectRequest.getCheckinComment();
            AbstractDocumentContent content = checkinObjectRequest.getContent();
            if(node != null && networkId != null && userId != null)
            {
	            Session cmisSession = getPublicApi(userId).getCMISSession(networkId);
	            Document object = (Document)cmisSession.getObject(node.getNodeId());
	            ObjectId objectId = object.checkIn(major, properties, content.getContentStream(), checkinComment);
	            String checkinId = objectId.getId();
	            node.setPwcId(null);
	            node.setCheckedOutBy(null);
	            response = new EventProcessorResponse("Checked in '" + checkinId, true, node, true);
	        }
	        else
	        {
	            response = new EventProcessorResponse("Failed to checkin object", false, null);
	        }
        }

        return response;
    }
}
