package org.alfresco.bm.publicapi.process;

import org.alfresco.bm.Services;
import org.alfresco.bm.event.selector.EventProcessorResponse;
import org.alfresco.bm.event.selector.EventSelector;
import org.alfresco.bm.publicapi.AbstractRepositoryEventSelectorProcessor;
import org.alfresco.bm.publicapi.data.DocumentNode;
import org.alfresco.bm.publicapi.data.Node;
import org.alfresco.bm.publicapi.data.Request;
import org.alfresco.bm.publicapi.factory.PublicApiFactory;
import org.apache.chemistry.opencmis.client.api.CmisObject;
import org.apache.chemistry.opencmis.client.api.Document;
import org.apache.chemistry.opencmis.client.api.ObjectId;
import org.apache.chemistry.opencmis.client.api.Session;

/**
 * Checkout an object.
 * 
 * @author steveglover
 *
 */
public class CheckoutObject extends AbstractRepositoryEventSelectorProcessor
{
    /**
     * @param services                access to collection services
     * @param publicApiFactory        a factory for creating connections to the public api
     * @param eventSelector           the event selector to use in selecting the next event
     */
    public CheckoutObject(Services services, PublicApiFactory publicApiFactory, EventSelector eventSelector)
    {
        super(services, publicApiFactory, eventSelector);
    }

    @Override
    public Object createDataObject(Object input, Object resp) throws Exception
    {
        CheckoutObjectRequest ret = null;

        if(input != null)
        {
            Request request = (Request)input;
	        DocumentNode node = getDocument(request, resp, canCheckOutNodeMatcher);
	        if(node != null)
	        {
	            logger.debug("CheckoutObject.createDataObject node = " + node);
	
	            String networkId = request.getNetworkId();
	            String userId = request.getRunAsUserId();
	            ret = new CheckoutObjectRequest(networkId, userId, node);
	        }
        }
        
        return ret;
    }

    @Override
    protected EventProcessorResponse processEventImpl(Object input) throws Exception
    {
        EventProcessorResponse response = null;

        if(input == null)
        {
        	 response = new EventProcessorResponse("Unable to checkout object, input is null", true, null);
        }
        else
        {
            CheckoutObjectRequest checkoutObjectRequest = (CheckoutObjectRequest)input;
            String userId = checkoutObjectRequest.getRunAsUserId();
            String networkId = checkoutObjectRequest.getNetworkId();
            DocumentNode node = checkoutObjectRequest.getNode();
            if(node != null && networkId != null && userId != null)
            {
	        	Session cmisSession = getPublicApi(userId).getCMISSession(networkId);
	
	            Document object = (Document)cmisSession.getObject(node.getNodeId());
	            ObjectId pwcId = object.checkOut();
	            if(pwcId != null)
	            {
	            	CmisObject pwc = cmisSession.getObject(pwcId);
	            	Node pwcNode = Node.createNode(pwc);
		            String pwcNodeId = pwcId.getId();
		            node.setPwcId(pwcNodeId);
		            node.setCheckedOutBy(userId);
		            response = new EventProcessorResponse("Checked out " + node + ", pwc is " + pwcNode, true, pwcNode, true);
	            }
	            else
	            {
	                response = new EventProcessorResponse("pwcId of checked out object is null, original node is " + node, false, null);
	            }
	        }
	        else
	        {
	            response = new EventProcessorResponse("Failed to checkout object", false, null);
	        }
        }

        return response;
    }
}
