package org.alfresco.bm.publicapi.process;

import java.util.HashSet;
import java.util.Set;
import java.util.UUID;

import org.alfresco.bm.Services;
import org.alfresco.bm.event.selector.EventProcessorResponse;
import org.alfresco.bm.event.selector.EventSelector;
import org.alfresco.bm.publicapi.AbstractRepositoryEventSelectorProcessor;
import org.alfresco.bm.publicapi.data.Node;
import org.alfresco.bm.publicapi.data.Request;
import org.alfresco.bm.publicapi.data.nodematchers.CanManipulateNodeMatcher;
import org.alfresco.bm.publicapi.factory.PublicApiFactory;
import org.springframework.social.alfresco.api.entities.Comment;

/**
 * 
 * @author steveglover
 *
 */
public class CreateComment extends AbstractRepositoryEventSelectorProcessor
{
    private CanManipulateNodeMatcher matcher = null;

    /**
     * @param services                access to collection services
     * @param publicApiFactory        a factory for creating connections to the public api
     * @param eventSelector           the event selector to use in selecting the next event
     */
    public CreateComment(Services services, PublicApiFactory publicApiFactory, EventSelector eventSelector)
    {
        super(services, publicApiFactory, eventSelector);
	    Set<String> requiredCmisTypes = new HashSet<String>();
	    Set<String> excludeCmisTypes = new HashSet<String>();
	    requiredCmisTypes.add("cmis:folder");
        this.matcher = new CanManipulateNodeMatcher(requiredCmisTypes, excludeCmisTypes);
    }


    @Override
    public Object createDataObject(Object input, Object resp) throws Exception
    {
        CreateCommentRequest ret = null;

        if(input != null)
        {
            Request request = (Request)input;

	        Node node = getNode(request, resp, matcher);
	        if(node != null)
	        {
	            logger.debug("CreateComment.createDataObject node = " + node);
	            
	            String comment = UUID.randomUUID().toString();
	            String networkId = request.getNetworkId();
	            String userId = request.getRunAsUserId();
	            ret = new CreateCommentRequest(networkId, userId, node, comment);
	        }
        }

        return ret;
    }

    @Override
    protected EventProcessorResponse processEventImpl(Object input) throws Exception
    {
        EventProcessorResponse response = null;

        if(input == null)
        {
        	 response = new EventProcessorResponse("Unable to checkout object, input is null", true, null);
        }
        else
        {
            CreateCommentRequest createCommentRequest = (CreateCommentRequest)input;
            String userId = createCommentRequest.getRunAsUserId();
            String networkId = createCommentRequest.getNetworkId();
            Node node = createCommentRequest.getNode();
            String comment = createCommentRequest.getComment();

	        if(node != null && comment != null && userId != null && networkId != null)
	        {
	            Comment ret = getPublicApi(userId).createComment(networkId, node.getNodeId(), comment);
	            response = new EventProcessorResponse("Created comment", true, ret, true);
	        }
	        else
	        {
	            response = new EventProcessorResponse("Failed to create comment, nodeId or comment is null", false, null);
	        }
        }

        return response;
    }
}