package org.alfresco.bm.publicapi.process;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;

import org.alfresco.bm.publicapi.data.DocumentContent;
import org.alfresco.bm.publicapi.data.FolderNode;
import org.alfresco.bm.publicapi.data.Request;
import org.apache.chemistry.opencmis.commons.enums.VersioningState;

public class CreateDocumentByParentIdRequest extends Request
{
    private FolderNode parentFolder;
    private VersioningState versioningState;
    private DocumentContent content;
    private Map<String, Serializable> properties;
    private String type;

    public CreateDocumentByParentIdRequest(String networkId, String runAsUserId, FolderNode parentFolder, VersioningState versioningState, String type,
            DocumentContent content, Map<String, Serializable> properties)
    {
        super(networkId, runAsUserId);
        this.parentFolder = parentFolder;
        this.type = type;
        this.versioningState = versioningState;
        this.content = content;
        if(properties == null)
        {
            properties = new HashMap<String, Serializable>();
        }
        this.properties = properties;
    }
    
    public String getType()
    {
        return type;
    }

    public FolderNode getParentFolder()
    {
        return parentFolder;
    }

    public VersioningState getVersioningState()
    {
        return versioningState;
    }

    public String getFilename()
    {
        return content.getFilename();
    }

    public DocumentContent getContent()
    {
        return content;
    }

    public Map<String, Serializable> getProperties()
    {
        return properties;
    }

    @Override
    public String toString()
    {
        return "CreateDocumentByParentIdRequest [parentFolder="
                + parentFolder + ", versioningState=" + versioningState
                + ", filename=" + content.getFilename() + ", content=" + content
                + ", properties=" + properties + ", type=" + type + "]";
    }
}
