package org.alfresco.bm.publicapi.process;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;

import org.alfresco.bm.publicapi.data.DocumentContent;
import org.alfresco.bm.publicapi.data.Path;
import org.alfresco.bm.publicapi.data.Request;
import org.apache.chemistry.opencmis.commons.enums.VersioningState;

public class CreateDocumentByParentPathRequest extends Request
{
    private Path parentFolderPath;
    private String parentFolderId;
    private VersioningState versioningState;
    private DocumentContent content;
    private Map<String, Serializable> properties;
    private String type;

    public CreateDocumentByParentPathRequest(String networkId, String runAsUserId, Path parentFolderPath, VersioningState versioningState,
            String type, DocumentContent content, Map<String, Serializable> properties)
    {
        super(networkId, runAsUserId);
        this.parentFolderPath = parentFolderPath;
        this.type = type;
        this.versioningState = versioningState;
        this.content = content;
        if(properties == null)
        {
            properties = new HashMap<String, Serializable>();
        }
        this.properties = properties;
    }

    public CreateDocumentByParentPathRequest(String networkId, String runAsUserId, String parentFolderId, VersioningState versioningState, String type,
            DocumentContent content, Map<String, Serializable> properties)
    {
        super(networkId, runAsUserId);
        this.parentFolderId = parentFolderId;
        this.type = type;
        this.versioningState = versioningState;
        this.content = content;
        if(properties == null)
        {
            properties = new HashMap<String, Serializable>();
        }
        this.properties = properties;
    }
    
    public String getType()
    {
        return type;
    }

    public Path getParentFolderPath()
    {
        return parentFolderPath;
    }

    public String getParentFolderId()
    {
        return parentFolderId;
    }

    public VersioningState getVersioningState()
    {
        return versioningState;
    }

    public String getFilename()
    {
        return content.getFilename();
    }

    public DocumentContent getContent()
    {
        return content;
    }

    public Map<String, Serializable> getProperties()
    {
        return properties;
    }

    @Override
    public String toString()
    {
        return "CreateDocumentRequest [parentFolderId=" + parentFolderId
                + ", versioningState=" + versioningState + ", filename="
                + content.getFilename() + ", content=" + content + ", properties="
                + properties + "]";
    }
}
