package org.alfresco.bm.publicapi.process;

import java.io.Serializable;
import java.util.Map;

import org.alfresco.bm.Services;
import org.alfresco.bm.event.selector.EventProcessorResponse;
import org.alfresco.bm.event.selector.EventSelector;
import org.alfresco.bm.publicapi.AbstractRepositoryEventSelectorProcessor;
import org.alfresco.bm.publicapi.data.CreateFolderRequestFactory;
import org.alfresco.bm.publicapi.data.FolderNode;
import org.alfresco.bm.publicapi.data.Path;
import org.alfresco.bm.publicapi.data.Request;
import org.alfresco.bm.publicapi.factory.PublicApiFactory;
import org.apache.chemistry.opencmis.client.api.Folder;
import org.apache.chemistry.opencmis.client.api.Session;
import org.apache.chemistry.opencmis.commons.PropertyIds;

/**
 * Create a folder.
 * 
 * @author steveglover
 *
 */
public class CreateFolder extends AbstractRepositoryEventSelectorProcessor
{
	private CreateFolderRequestFactory requestFactory;

    /**
     * 
     * @param services                access to collection services
     * @param publicApiFactory        a factory for creating connections to the public api
     * @param eventSelector           the event selector to use in selecting the next event
     */
    public CreateFolder(Services services, PublicApiFactory publicApiFactory, EventSelector eventSelector, CreateFolderRequestFactory requestFactory)
    {
        super(services, publicApiFactory, eventSelector);
        this.requestFactory = requestFactory;
    }

    @Override
    public Object createDataObject(Object input, Object resp) throws Exception
    {
        CreateFolderWithParentIdRequest ret = null;

        if(input != null)
        {
            Request request = (Request)input;

	        FolderNode parentFolder = getFolder(request, resp, canCreateFolderMatcher);
	        if(parentFolder != null)
	        {
	            logger.debug("CreateFolder.createDataObject parentFolder = " + parentFolder);
	            ret = requestFactory.create(request.getNetworkId(), request.getRunAsUserId(), parentFolder);
	        }
        }

        return ret;
    }

    @Override
    protected EventProcessorResponse processEventImpl(Object input) throws Exception
    {
        EventProcessorResponse response = null;

        if(input == null)
        {
        	 response = new EventProcessorResponse("Unable to create folder, input is null", true, null);
        }
        else
        {
            Request request = (Request)input;
            String userId = null;
            String networkId = null;
            FolderNode parentFolder = null;
            Path parentFolderPath = null;
            String type = null;
            Map<String, Serializable> properties = null;

        	if(request instanceof CreateFolderWithParentIdRequest)
	        {
	            CreateFolderWithParentIdRequest req = (CreateFolderWithParentIdRequest)request;
	            parentFolder = req.getParentFolder();
	            userId = req.getRunAsUserId();
	            networkId = req.getNetworkId();
	            type = req.getType();
	            properties = req.getProperties();
	        }
	        else if(request instanceof CreateFolderWithParentPathRequest)
	        {
	            CreateFolderWithParentPathRequest req = (CreateFolderWithParentPathRequest)request;
	            parentFolderPath = req.getParentFolderPath();
	            userId = req.getRunAsUserId();
	            networkId = req.getNetworkId();
	            type = req.getType();
	            properties = req.getProperties();
	        }
	        else
	        {
	            throw new IllegalArgumentException("Invalid request for createFolder: " + request);
	        }
	
	        properties.put(PropertyIds.OBJECT_TYPE_ID, type);
	
	        if(parentFolder == null && parentFolderPath == null)
	        {
	            response = new EventProcessorResponse("Failed to create folder node: must specify parentFolder or parentFolderPath", false, null);
	        }
	        else
	        {
	            Session cmisSession = getPublicApi(userId).getCMISSession(networkId);
	
	            Folder parent = null;
	            if(parentFolder != null)
	            {
	                parent = (Folder)cmisSession.getObject(parentFolder.getNodeId());
	            }
	            else if(parentFolderPath != null)
	            {
	                parent = (Folder)cmisSession.getObjectByPath(parentFolderPath.getPath());
	            }
	            
	            Folder folder = parent.createFolder(properties);
	            FolderNode res = (FolderNode)FolderNode.createNode(folder);
	            response = new EventProcessorResponse("Created folder '" + res.getNodeId(), true, res, true);
	        }
        }

        return response;
    }
}
