package org.alfresco.bm.publicapi.process;

import java.util.HashSet;
import java.util.Set;

import org.alfresco.bm.Services;
import org.alfresco.bm.event.selector.EventProcessorResponse;
import org.alfresco.bm.event.selector.EventSelector;
import org.alfresco.bm.publicapi.AbstractRepositoryEventSelectorProcessor;
import org.alfresco.bm.publicapi.data.Node;
import org.alfresco.bm.publicapi.data.Request;
import org.alfresco.bm.publicapi.data.nodematchers.CanManipulateNodeMatcher;
import org.alfresco.bm.publicapi.factory.PublicApiFactory;
import org.springframework.social.alfresco.api.entities.Rating;

public class CreateNodeRating extends AbstractRepositoryEventSelectorProcessor
{
    private CanManipulateNodeMatcher matcher = null;
    
    /**
     * @param services                access to collection services
     * @param publicApiFactory        a factory for creating connections to the public api
     * @param eventSelector           the event selector to use in selecting the next event
     */
    public CreateNodeRating(Services services, PublicApiFactory publicApiFactory, EventSelector eventSelector)
    {
        super(services, publicApiFactory, eventSelector);
	    Set<String> requiredCmisTypes = new HashSet<String>();
	    Set<String> excludeCmisTypes = new HashSet<String>();
	    requiredCmisTypes.add("cmis:folder");
        this.matcher = new CanManipulateNodeMatcher(requiredCmisTypes, excludeCmisTypes);
    }

    @Override
    public Object createDataObject(Object input, Object resp) throws Exception
    {
        CreateNodeRatingRequest ret = null;

        if(input != null)
        {
            Request request = (Request)input;

	        Node node = getNode(request, resp, matcher);
	        if(node != null)
	        {
	            logger.debug("CreateNodeRating.createDataObject node = " + node);
	
	            String networkId = request.getNetworkId();
	            String userId = request.getRunAsUserId();
	            String ratingId = Rating.LIKES;
	            Object rating = Boolean.TRUE;
	    
	            ret = new CreateNodeRatingRequest(networkId, userId, node, ratingId, rating);
	        }
        }

        return ret;
    }

    @Override
    protected EventProcessorResponse processEventImpl(Object input) throws Exception
    {
        EventProcessorResponse response = null;

        if(input == null)
        {
        	response = new EventProcessorResponse("Unable to create node rating, input is null", true, null);
        }
        else
        {
            CreateNodeRatingRequest createNodeRatingRequest = (CreateNodeRatingRequest)input;
            String userId = createNodeRatingRequest.getRunAsUserId();
            String networkId = createNodeRatingRequest.getNetworkId();
            Node node = createNodeRatingRequest.getNode();
            String ratingId = createNodeRatingRequest.getRatingId();
            Object rating = createNodeRatingRequest.getRating();
            
            if(ratingId == null)
            {
            	response = new EventProcessorResponse("Failed to add node rating: no rating id supplied", false, null);
	        }
	        else if(rating == null)
	        {
	            response = new EventProcessorResponse("Failed to add node rating: no rating supplied", false, null);
	        }
	        else if(userId == null)
	        {
	            response = new EventProcessorResponse("Failed to add node rating: no userId supplied", false, null);
	        }
	        else if(networkId == null)
	        {
	            response = new EventProcessorResponse("Failed to add node rating: no networkId supplied", false, null);
	        }
	        else if(node == null)
	        {
	            response = new EventProcessorResponse("Failed to add node rating: no node supplied", false, null);
	        }
	        else
	        {
	            if(ratingId.equals(Rating.LIKES))
	            {
	                Boolean booleanRating = (Boolean)rating;
	                Rating nodeRating = getPublicApi(userId).rateNode(networkId, node.getNodeId(), booleanRating.booleanValue());
	                response = new EventProcessorResponse("Added node rating: \n" + "   Response: " + nodeRating, true, node, true);
	            }
	            else if(ratingId.equals(Rating.STARS))
	            {
	                Integer intRating = (Integer)rating;
	                Rating nodeRating = getPublicApi(userId).rateNode(networkId, node.getNodeId(), intRating.intValue());
	                response = new EventProcessorResponse("Added node rating: \n" + "   Response: " + nodeRating, true, node, true);
	            }
	            else
	            {
	                response = new EventProcessorResponse("Failed to add node rating: invalid rating id", false, null);
	            }
	        }
        }
            
        return response;
    }
    
}
