package org.alfresco.bm.publicapi.process;

import java.util.HashSet;
import java.util.Set;
import java.util.UUID;

import org.alfresco.bm.Services;
import org.alfresco.bm.event.selector.EventProcessorResponse;
import org.alfresco.bm.event.selector.EventSelector;
import org.alfresco.bm.publicapi.AbstractRepositoryEventSelectorProcessor;
import org.alfresco.bm.publicapi.data.Node;
import org.alfresco.bm.publicapi.data.Request;
import org.alfresco.bm.publicapi.data.nodematchers.CanManipulateNodeMatcher;
import org.alfresco.bm.publicapi.factory.PublicApiFactory;

public class CreateNodeTag extends AbstractRepositoryEventSelectorProcessor
{
    private CanManipulateNodeMatcher matcher = null;
        
    /**
     * @param services                access to collection services
     * @param publicApiFactory        a factory for creating connections to the public api
     * @param eventSelector           the event selector to use in selecting the next event
     */
    public CreateNodeTag(Services services, PublicApiFactory publicApiFactory, EventSelector eventSelector)
    {
        super(services, publicApiFactory, eventSelector);
	    Set<String> requiredCmisTypes = new HashSet<String>();
	    Set<String> excludeCmisTypes = new HashSet<String>();
	    requiredCmisTypes.add("cmis:folder");
        this.matcher = new CanManipulateNodeMatcher(requiredCmisTypes, excludeCmisTypes);
    }

    @Override
    public Object createDataObject(Object input, Object resp) throws Exception
    {
        CreateNodeTagRequest ret = null;

        if(input != null)
        {
            Request request = (Request)input;
            
	        Node node = getNode(request, resp, matcher);
	        if(node != null)
	        {
	            logger.debug("CreateNodeTag.createDataObject node = " + node);
	
	            String tag = UUID.randomUUID().toString();
	            String userId = request.getRunAsUserId();
	            String networkId = request.getNetworkId();
	    
	            ret = new CreateNodeTagRequest(networkId, userId, node, tag);
	        }
        }
        
        return ret;
    }

    @Override
    protected EventProcessorResponse processEventImpl(Object input) throws Exception
    {
        EventProcessorResponse response = null;

        if(input == null)
        {
        	 response = new EventProcessorResponse("Unable to add node tag, input is null", true, null);
        }
        else
        {
            CreateNodeTagRequest createNodeTagRequest = (CreateNodeTagRequest)input;
            String userId = createNodeTagRequest.getRunAsUserId();
            String networkId = createNodeTagRequest.getNetworkId();
            Node node = createNodeTagRequest.getNode();
            String tag = createNodeTagRequest.getTag();

            if(node != null && userId != null && networkId != null && tag != null && !tag.equals(""))
            {
	            getPublicApi(userId).addTagToNode(networkId, node.getNodeId(), tag);
	            response = new EventProcessorResponse("Added tag " + tag + " to node " + node.getNodeId(), true, node, true);
	        }
	        else
	        {
	            response = new EventProcessorResponse("Skipping add node rating: no node rating supplied.", false, null);
	        }
        }
        
        return response;
    }

}
