package org.alfresco.bm.publicapi.process;

import java.util.ArrayList;
import java.util.List;

import org.alfresco.bm.Services;
import org.alfresco.bm.event.AbstractEventProcessor;
import org.alfresco.bm.event.Event;
import org.alfresco.bm.event.EventResult;
import org.alfresco.bm.event.selector.EventSelector;
import org.alfresco.bm.user.UserData;

/**
 * Initiate a public api "session".
 * 
 * @author steveglover
 *
 */
public class CreateSession extends AbstractEventProcessor
{
    private Services services;
    private EventSelector eventSelector;
    
    /**
     * Creates a new instance.
     * 
     * @param services                access to collection services
     * @param eventSelector           the event selector to use in selecting the next event
     */
    public CreateSession(Services services, EventSelector eventSelector)
    {
        super();
        this.services = services;
        this.eventSelector = eventSelector;
    }

    public EventResult processEvent(Event event) throws Exception
    {
    	boolean success = false;
        String msg = null;
        List<Event> nextEvents = new ArrayList<Event>();

        UserData runAsUser = services.getUserDataService().randomUser();
        if(runAsUser != null)
        {
            String networkId = runAsUser.getDomain();
            if(networkId != null)
            {
                BasicRequest request = new BasicRequest(networkId, runAsUser.getUsername());
                Event initialPublicApiEvent = eventSelector.nextEvent(request, null);
                if(initialPublicApiEvent != null)
                {
                    // Start a recorded session with no initial data
                    String sessionId = services.getSessionService().startSession(null);
                    initialPublicApiEvent.setSessionId(sessionId);

                    nextEvents.add(initialPublicApiEvent);
                    msg = "Created initial public api event.";
                    success = true;
                }
                else
                {
                    msg = "Failed to create initial public api event.";
                }
            }
            else
            {
                msg = "Failed to create initial public api event, networkId is null";
            }
        }
        else
        {
            msg = "Failed to create initial public api event, no users found";
        }

        // Return messages + next events
        return new EventResult(msg, nextEvents, success);
    }
}
