package org.alfresco.bm.publicapi.process;

import org.alfresco.bm.Services;
import org.alfresco.bm.event.selector.EventProcessorResponse;
import org.alfresco.bm.event.selector.EventSelector;
import org.alfresco.bm.publicapi.AbstractRepositoryEventSelectorProcessor;
import org.alfresco.bm.publicapi.data.Request;
import org.alfresco.bm.publicapi.factory.PublicApiFactory;
import org.alfresco.bm.site.SiteMember;
import org.alfresco.bm.site.SiteRole;
import org.alfresco.bm.user.UserData;
import org.springframework.http.HttpStatus;
import org.springframework.social.alfresco.api.entities.Role;
import org.springframework.social.alfresco.api.entities.Site;
import org.springframework.social.alfresco.connect.exception.AlfrescoException;

public class CreateSiteMember extends AbstractRepositoryEventSelectorProcessor
{
	public CreateSiteMember(Services services, PublicApiFactory publicApiFactory, EventSelector eventSelector)
	{
		super(services, publicApiFactory, eventSelector);
	}

	@Override
	public Object createDataObject(Object input, Object resp) throws Exception
	{
		CreateSiteMemberRequest ret = null;

		if(input != null)
		{
			Request request = (Request)input;
			String runAsUserId = request.getRunAsUserId();
			String networkId = request.getNetworkId();

			// try to find a site in the response from the previous event created by the runAsUser.
			String siteId = null;
			Site site = findSite(request, resp, runAsUserId);
			if(site == null)
			{
				// no luck, grab a random site from the mirror that has been created by the runAsUser
				siteId = randomSiteForCreator(runAsUserId);
			}
			else
			{
				// got it
				siteId = site.getId();
			}
			
			if(siteId != null)
			{
				UserData person = services.getUserDataService().randomUser(networkId);
				Role role = randomRole();
	
				ret = new CreateSiteMemberRequest(networkId, runAsUserId, siteId, person.getUsername(), role);
			}
		}

		return ret;
	}

	@Override
	protected EventProcessorResponse processEventImpl(Object input) throws Exception
	{
        EventProcessorResponse response = null;

        if(input == null)
        {
        	 response = new EventProcessorResponse("Unable to create site member, input is null", true, null);
        }
        else
        {
            CreateSiteMemberRequest createSiteMemberRequest = (CreateSiteMemberRequest)input;
            String userId = createSiteMemberRequest.getRunAsUserId();
            String networkId = createSiteMemberRequest.getNetworkId();
            String siteId = createSiteMemberRequest.getSiteId();
            String personId = createSiteMemberRequest.getUserId();
            Role role = createSiteMemberRequest.getRole();

	        // Check the input
	        if (siteId == null || personId == null || role == null || userId == null || networkId == null)
	        {
				response = new EventProcessorResponse("Invalid site member request: " + createSiteMemberRequest, false, null);
	        }
	        else
	        {
		        SiteMember siteMember = new SiteMember(personId, siteId, null, SiteRole.valueOf(role.toString()));
		        try
		        {
		            getPublicApi(userId).addMember(networkId, siteId, personId, role);
		            services.getSiteDataService().setSiteMemberCreated(siteMember.getSiteId(), siteMember.getUsername(), siteMember.getRole(), Boolean.TRUE);
		            
		            response = new EventProcessorResponse("Added site member", true, siteMember);
		        }
		        catch(AlfrescoException e)
		        {
		            if(e.getStatusCode().equals(HttpStatus.CONFLICT))
		            {
		                // already a member
		                services.getSiteDataService().setSiteMemberCreated(siteMember.getSiteId(), siteMember.getUsername(), siteMember.getRole(), Boolean.TRUE);
	
		    			response = new EventProcessorResponse("Added site member" + siteId, true, siteMember);
		            }
		            else
		            {
		                throw e;
		            }
		        }
	        }
        }

		return response;
	}
}