package org.alfresco.bm.publicapi.process;

import org.alfresco.bm.Services;
import org.alfresco.bm.event.selector.EventProcessorResponse;
import org.alfresco.bm.event.selector.EventSelector;
import org.alfresco.bm.publicapi.AbstractRepositoryEventSelectorProcessor;
import org.alfresco.bm.publicapi.data.DocumentNode;
import org.alfresco.bm.publicapi.data.Request;
import org.alfresco.bm.publicapi.factory.PublicApiFactory;
import org.apache.chemistry.opencmis.client.api.Document;
import org.apache.chemistry.opencmis.client.api.Session;

public class DeleteContent extends AbstractRepositoryEventSelectorProcessor
{
    /**
     * @param services                access to collection services
     * @param publicApiFactory        a factory for creating connections to the public api
     * @param eventSelector           the event selector to use in selecting the next event
     */
    public DeleteContent(Services services, PublicApiFactory publicApiFactory, EventSelector eventSelector)
    {
        super(services, publicApiFactory, eventSelector);
    }

    @Override
    public Object createDataObject(Object input, Object resp) throws Exception
    {
        DeleteContentRequest ret = null;

        if(input != null)
        {
            Request request = (Request)input;
            
	        DocumentNode node = getDocument(request, resp, canDeleteContentMatcher);
	        if(node != null)
	        {
	            logger.debug("DeleteContent.createDataObject node = " + node);
	
	            String networkId = request.getNetworkId();
	            String userId = request.getRunAsUserId();
	            ret = new DeleteContentRequest(networkId, userId, node);
	        }
        }
        
        return ret;
    }

    @Override
    protected EventProcessorResponse processEventImpl(Object input) throws Exception
    {
        EventProcessorResponse response = null;

        if(input == null)
        {
        	 response = new EventProcessorResponse("Unable to delete content, input is null", true, null);
        }
        else
        {
            DeleteContentRequest deleteContentRequest = (DeleteContentRequest)input;
            String userId = deleteContentRequest.getRunAsUserId();
            String networkId = deleteContentRequest.getNetworkId();
            DocumentNode node = deleteContentRequest.getNode();
            if(node != null && userId != null && networkId != null)
            {
	            Session cmisSession = getPublicApi(userId).getCMISSession(networkId);
	            Document doc = (Document)cmisSession.getObject(node.getNodeId());
	            doc.deleteContentStream();
	
	            response = new EventProcessorResponse("Deleted content '" + doc.getId(), true, node, true);
	        }
	        else
	        {
	            response = new EventProcessorResponse("Failed to delete content, nodeId is null", false, null);
	        }
        }
        
        return response;

    }
}
