package org.alfresco.bm.publicapi.process;

import org.alfresco.bm.Services;
import org.alfresco.bm.event.selector.EventProcessorResponse;
import org.alfresco.bm.event.selector.EventSelector;
import org.alfresco.bm.publicapi.AbstractRepositoryEventSelectorProcessor;
import org.alfresco.bm.publicapi.data.Node;
import org.alfresco.bm.publicapi.data.Request;
import org.alfresco.bm.publicapi.factory.PublicApiFactory;
import org.apache.chemistry.opencmis.client.api.CmisObject;
import org.apache.chemistry.opencmis.client.api.Session;

/**
 * Delete node event processor.
 * 
 * @author steveglover
 *
 */
public class DeleteNode extends AbstractRepositoryEventSelectorProcessor
{
    /**
     * @param services                access to collection services
     * @param publicApiFactory        a factory for creating connections to the public api
     * @param eventSelector           the event selector to use in selecting the next event
     */
    public DeleteNode(Services services, PublicApiFactory publicApiFactory, EventSelector eventSelector)
    {
        super(services, publicApiFactory, eventSelector);
    }

    @Override
    public Object createDataObject(Object input, Object resp) throws Exception
    {
        DeleteNodeRequest  ret = null;

        if(input != null)
        {
            Request request = (Request)input;

	        Node node = getNode(request, resp, canDeleteNodeMatcher);
	        if(node != null)
	        {
	            logger.debug("DeleteNode.createDataObject node = " + node);
	
	            boolean allVersions = Boolean.TRUE;
	            String networkId = request.getNetworkId();
	            String userId = request.getRunAsUserId();
	            ret = new DeleteNodeRequest(networkId, userId, node, allVersions);
	        }
        }
        
        return ret;
    }

    @Override
    protected EventProcessorResponse processEventImpl(Object input) throws Exception
    {
        EventProcessorResponse response = null;

        if(input == null)
        {
        	 response = new EventProcessorResponse("Unable to checkout object, input is null", true, null);
        }
        else
        {
            DeleteNodeRequest deleteNodeRequest = (DeleteNodeRequest)input;
            String userId = deleteNodeRequest.getRunAsUserId();
            String networkId = deleteNodeRequest.getNetworkId();
            Node node = deleteNodeRequest.getNode();
            Boolean allVersions = deleteNodeRequest.getAllVersions();
            if(allVersions == null)
            {
                allVersions = Boolean.TRUE;
            }
            
	        if(node != null && userId != null && networkId != null)
	        {
	            Session cmisSession = getPublicApi(userId).getCMISSession(networkId);
	            CmisObject object = cmisSession.getObject(node.getNodeId());
	            object.delete(allVersions);
	
	            response = new EventProcessorResponse("Deleted node '" + object.getId(), true, null);
	        }
	        else
	        {
	            response = new EventProcessorResponse("Failed to delete node, nodeId is null", false, null);
	        }
        }

        return response;
    }
}
