package org.alfresco.bm.publicapi.process;

import java.util.List;

import org.alfresco.bm.Services;
import org.alfresco.bm.event.selector.EventProcessorResponse;
import org.alfresco.bm.event.selector.EventSelector;
import org.alfresco.bm.publicapi.AbstractRepositoryEventSelectorProcessor;
import org.alfresco.bm.publicapi.data.DocumentNode;
import org.alfresco.bm.publicapi.data.Node;
import org.alfresco.bm.publicapi.data.NodesContainer;
import org.alfresco.bm.publicapi.data.Request;
import org.alfresco.bm.publicapi.factory.PublicApiFactory;
import org.apache.chemistry.opencmis.client.api.CmisObject;
import org.apache.chemistry.opencmis.client.api.Document;
import org.apache.chemistry.opencmis.client.api.Session;

/**
 * Get all versions of an object.
 * 
 * @author steveglover
 *
 */
public class GetAllVersions extends AbstractRepositoryEventSelectorProcessor
{
    /**
     * @param services                access to collection services
     * @param publicApiFactory        a factory for creating connections to the public api
     * @param eventSelector           the event selector to use in selecting the next event
     */
    public GetAllVersions(Services services, PublicApiFactory publicApiFactory, EventSelector eventSelector)
    {
        super(services, publicApiFactory, eventSelector);
    }

    @Override
    public Object createDataObject(Object input, Object resp) throws Exception
    {
        GetAllVersionsRequest ret = null;

        if(input != null)
        {
            Request request = (Request)input;
            
	        DocumentNode node = getDocument(request, resp, canGetAllVersionsNodeMatcher);
	        if(node != null)
	        {
	            logger.debug("GetAllVersions.createDataObject node = " + node);
	
	            String networkId = request.getNetworkId();
	            String userId = request.getRunAsUserId();
	            int skipCount = 0;
	            int maxItems = 10;
	            ret = new GetAllVersionsRequest(networkId, userId, node, skipCount, maxItems);
	        }
        }
        
        return ret;
    }

    @Override
    protected EventProcessorResponse processEventImpl(Object input) throws Exception
    {
        EventProcessorResponse response = null;

        if(input == null)
        {
        	 response = new EventProcessorResponse("Unable to create site member, input is null", true, null);
        }
        else
        {
        	GetAllVersionsRequest getAllVersionsRequest = (GetAllVersionsRequest)input;
            String userId = getAllVersionsRequest.getRunAsUserId();
            String networkId = getAllVersionsRequest.getNetworkId();
        	Node node = getAllVersionsRequest.getNode();

	        if(networkId != null && userId != null)
	        {
	        	Session cmisSession = getPublicApi(userId).getCMISSession(networkId);
	        	CmisObject obj = cmisSession.getObject(node.getNodeId());
	        	if(obj instanceof Document)
	        	{
	        		Document document = (Document)obj;
		        	List<Document> allVersions = document.getAllVersions();
		            NodesContainer nodes = new NodesContainer(allVersions);
		
		            response = new EventProcessorResponse("Got all versions for " + node, true, nodes, true);
	        	}
	        	else
	        	{
	                response = new EventProcessorResponse("Failed to get all versions for " + node + ", it is not a document", false, null);
	        	}
	        }
	        else
	        {
	            response = new EventProcessorResponse("Failed to get all versions for " + node, false, null);
	        }
        }

        return response;
    }

}
