package org.alfresco.bm.publicapi.process;

import org.alfresco.bm.Services;
import org.alfresco.bm.event.selector.EventProcessorResponse;
import org.alfresco.bm.event.selector.EventSelector;
import org.alfresco.bm.publicapi.AbstractRepositoryEventSelectorProcessor;
import org.alfresco.bm.publicapi.data.FolderNode;
import org.alfresco.bm.publicapi.data.NodesContainer;
import org.alfresco.bm.publicapi.data.Request;
import org.alfresco.bm.publicapi.factory.PublicApiFactory;
import org.apache.chemistry.opencmis.client.api.Document;
import org.apache.chemistry.opencmis.client.api.ItemIterable;
import org.apache.chemistry.opencmis.client.api.Session;
import org.apache.chemistry.opencmis.commons.enums.IncludeRelationships;

/**
 * Get checked out documents.
 * 
 * @author steveglover
 *
 */
public class GetCheckedOutDocs extends AbstractRepositoryEventSelectorProcessor
{
    /**
     * @param services                access to collection services
     * @param publicApiFactory        a factory for creating connections to the public api
     * @param eventSelector           the event selector to use in selecting the next event
     */
    public GetCheckedOutDocs(Services services, PublicApiFactory publicApiFactory, EventSelector eventSelector)
    {
        super(services, publicApiFactory, eventSelector);
    }

    @Override
    public Object createDataObject(Object input, Object resp) throws Exception
    {
        GetCheckedOutDocsRequest ret = null;

        if(input != null)
        {
            Request request = (Request)input;
        	FolderNode folder = null;

            logger.debug("GetCheckedoutDocs.createDataObject folder = " + folder);

            String networkId = request.getNetworkId();
            String userId = request.getRunAsUserId();
            int skipCount = 0;
            int maxItems = 10;
            String filter = null;
            String orderBy = null;
            Boolean includeAllowableActions = true;
            IncludeRelationships includeRelationships = IncludeRelationships.BOTH;
            String renditionFilter = null;
            ret = new GetCheckedOutDocsRequest(networkId, userId, skipCount, maxItems, folder, filter, orderBy,
            		includeAllowableActions, includeRelationships, renditionFilter);
        }
        
        return ret;
    }

    @Override
    protected EventProcessorResponse processEventImpl(Object input) throws Exception
    {
        EventProcessorResponse response = null;

        if(input == null)
        {
        	 response = new EventProcessorResponse("Unable to create site member, input is null", true, null);
        }
        else
        {
            GetCheckedOutDocsRequest getCheckedOutDocsRequest = (GetCheckedOutDocsRequest)input;
            String userId = getCheckedOutDocsRequest.getRunAsUserId();
            String networkId = getCheckedOutDocsRequest.getNetworkId();
            
	        if(networkId != null && userId != null)
	        {
	        	Session cmisSession = getPublicApi(userId).getCMISSession(networkId);
	
	            ItemIterable<Document> checkedOutDocs = cmisSession.getCheckedOutDocs();
	            NodesContainer nodes = new NodesContainer(checkedOutDocs);
	
	            response = new EventProcessorResponse("Got checked out docs " + nodes, true, nodes, true);
	        }
	        else
	        {
	            response = new EventProcessorResponse("Failed to get checked out documents " +networkId , false, null);
	        }
        }

        return response;
    }
}
