package org.alfresco.bm.publicapi.process;

import java.util.Map;

import org.alfresco.bm.Services;
import org.alfresco.bm.event.selector.EventProcessorResponse;
import org.alfresco.bm.event.selector.EventSelector;
import org.alfresco.bm.publicapi.AbstractRepositoryEventSelectorProcessor;
import org.alfresco.bm.publicapi.data.Node;
import org.alfresco.bm.publicapi.data.Request;
import org.alfresco.bm.publicapi.factory.PublicApiFactory;
import org.springframework.social.alfresco.api.entities.AlfrescoList;
import org.springframework.social.alfresco.api.entities.Comment;

/**
 * Get comments event selector.
 * 
 * @author steveglover
 *
 */
public class GetComments extends AbstractRepositoryEventSelectorProcessor
{
    /**
     * @param services                access to collection services
     * @param publicApiFactory        a factory for creating connections to the public api
     * @param eventSelector           the event selector to use in selecting the next event
     */
    public GetComments(Services services, PublicApiFactory publicApiFactory, EventSelector eventSelector)
    {
        super(services, publicApiFactory, eventSelector);
    }
    
    @Override
    public Object createDataObject(Object input, Object resp) throws Exception
    {
        GetCommentsRequest ret = null;

        if(input != null)
        {
            Request request = (Request)input;

	        Node node = getNode(request, resp, currentNodeMatcher);
	        if(node != null)
	        {
	            logger.debug("GetComments.createDataObject node = " + node);

	            String networkId = request.getNetworkId();
	            String userId = request.getRunAsUserId();
	            // TODO more random...
	            int skipCount = 0;
	            int maxItems = Integer.MAX_VALUE;

	            ret = new GetCommentsRequest(networkId, userId, node, skipCount, maxItems);
	        }
        }

        return ret;
    }

    @Override
    protected EventProcessorResponse processEventImpl(Object input) throws Exception
    {
        EventProcessorResponse response = null;

        if(input == null)
        {
        	 response = new EventProcessorResponse("Unable to get comments, input is null", true, null);
        }
        else
        {
            GetCommentsRequest getCommentsRequest = (GetCommentsRequest)input;
            String userId = getCommentsRequest.getRunAsUserId();
            String networkId = getCommentsRequest.getNetworkId();
            Node node = getCommentsRequest.getNode();
            Integer skipCount = getCommentsRequest.getSkipCount();
            Integer maxItems = getCommentsRequest.getMaxItems();

            if(node != null && userId != null && networkId != null)
            {
	            Map<String, String> parameters = new Parameters().addParameter("skipCount", skipCount).
	                    addParameter("maxItems", maxItems).toMap();
	
	            AlfrescoList<Comment> comments = getPublicApi(userId).getComments(networkId, node.getNodeId(), parameters);
	            node.setComments(comments);
	            response = new EventProcessorResponse("Got comments: \n" + "   Response: " + comments, true, node, true);
            }
	        else
	        {
	            response = new EventProcessorResponse("Skipping get comments: no node supplied.", true, null);
	        }
        }

        return response;
    }
}
