package org.alfresco.bm.publicapi.process;

import org.alfresco.bm.Services;
import org.alfresco.bm.event.selector.EventProcessorResponse;
import org.alfresco.bm.event.selector.EventSelector;
import org.alfresco.bm.publicapi.AbstractRepositoryEventSelectorProcessor;
import org.alfresco.bm.publicapi.data.AbstractDocumentContent;
import org.alfresco.bm.publicapi.data.DocumentNode;
import org.alfresco.bm.publicapi.data.Request;
import org.alfresco.bm.publicapi.factory.PublicApiFactory;
import org.apache.chemistry.opencmis.client.api.Document;
import org.apache.chemistry.opencmis.client.api.Session;
import org.apache.chemistry.opencmis.commons.data.ContentStream;

/**
 * Get content event selector.
 * 
 * @author steveglover
 *
 */
public class GetContent extends AbstractRepositoryEventSelectorProcessor
{
    /**
     * @param services                access to collection services
     * @param publicApiFactory        a factory for creating connections to the public api
     * @param eventSelector           the event selector to use in selecting the next event
     */
    public GetContent(Services services, PublicApiFactory publicApiFactory, EventSelector eventSelector)
    {
        super(services, publicApiFactory, eventSelector);
    }

    @Override
    public Object createDataObject(Object input, Object resp) throws Exception
    {
        GetContentRequest  ret = null;

        if(input != null)
        {
            Request request = (Request)input;
            
	        DocumentNode node = getDocument(request, resp);
	        if(node != null)
	        {
	            logger.debug("GetContent.createDataObject node = " + node);
	
	            String networkId = request.getNetworkId();
	            String userId = request.getRunAsUserId();
	            ret = new GetContentRequest(networkId, userId, node);
	        }
        }
        
        return ret;
    }

    @Override
    protected EventProcessorResponse processEventImpl(Object input) throws Exception
    {
        EventProcessorResponse response = null;

        if(input == null)
        {
        	 response = new EventProcessorResponse("Unable to get content, input is null", true, null);
        }
        else
        {
            GetContentRequest getContentRequest = (GetContentRequest)input;
            String userId = getContentRequest.getRunAsUserId();
            String networkId = getContentRequest.getNetworkId();
            DocumentNode node = getContentRequest.getNode();

	        if(node != null && userId != null && networkId != null)
	        {
	            Session cmisSession = getPublicApi(userId).getCMISSession(networkId);
	            Document document = (Document)cmisSession.getObject(node.getNodeId());
	            ContentStream stream = document.getContentStream();
	            AbstractDocumentContent documentContent = AbstractDocumentContent.getContent(stream);

	            response = new EventProcessorResponse("Got content for '" + node + "'.", true, documentContent, true);
	        }
	        else
	        {
	            response = new EventProcessorResponse("Skipping get content: no nodeId supplied.", true, null);
	        }
        }

        return response;
    }
}
