package org.alfresco.bm.publicapi.process;

import java.util.List;
import java.util.Set;

import org.alfresco.bm.Services;
import org.alfresco.bm.event.selector.EventProcessorResponse;
import org.alfresco.bm.event.selector.EventSelector;
import org.alfresco.bm.publicapi.AbstractRepositoryEventSelectorProcessor;
import org.alfresco.bm.publicapi.data.FolderNode;
import org.alfresco.bm.publicapi.data.Request;
import org.alfresco.bm.publicapi.factory.PublicApiFactory;
import org.apache.chemistry.opencmis.client.api.FileableCmisObject;
import org.apache.chemistry.opencmis.client.api.Tree;
import org.apache.chemistry.opencmis.commons.enums.IncludeRelationships;

/**
 * Get descendants event processor.
 * 
 * @author steveglover
 *
 */
public class GetDescendants extends AbstractRepositoryEventSelectorProcessor
{
    public final static int DEFAULT_DEPTH = 1;

    /**
     * @param services                access to collection services
     * @param publicApiFactory        a factory for creating connections to the public api
     * @param eventSelector           the event selector to use in selecting the next event
     */
    public GetDescendants(Services services, PublicApiFactory publicApiFactory, EventSelector eventSelector)
    {
        super(services, publicApiFactory, eventSelector);
    }
    
    @Override
    public Object createDataObject(Object input, Object resp) throws Exception
    {
        GetDescendantsRequest ret = null;

        if(input != null)
        {
            Request request = (Request)input;

	        FolderNode folder = getFolder(request, resp, canGetDescendantsMatcher);
	        
	        if(folder != null)
	        {
	            logger.debug("GetDescendants.createDataObject folder = " + folder);
	            
	            String networkId = request.getNetworkId();
	            String userId = request.getRunAsUserId();
	            Integer depth = 2; // TODO
	            IncludeRelationships includeRelationships = IncludeRelationships.BOTH;
	            Boolean includeAcls = Boolean.TRUE;
	            Set<String> propertyFilter = null;
	            Boolean includePolicies = Boolean.TRUE;
	            
	            ret = new GetDescendantsRequest(networkId, userId, folder, depth, includeRelationships, includeAcls, propertyFilter, includePolicies);
	        }
        }
        
        return ret;
    }

    @Override
    protected EventProcessorResponse processEventImpl(Object input) throws Exception
    {
        EventProcessorResponse response = null;

        if(input == null)
        {
        	 response = new EventProcessorResponse("Unable to get descendants, input is null", true, null);
        }
        else
        {
            GetDescendantsRequest getDescendantsRequest = (GetDescendantsRequest)input;
            String userId = getDescendantsRequest.getRunAsUserId();
            String networkId = getDescendantsRequest.getNetworkId();
            FolderNode folder = getDescendantsRequest.getFolder();
            Integer depth = getDescendantsRequest.getDepth();
            if(depth == null)
            {
                depth = DEFAULT_DEPTH;
            }
            IncludeRelationships includeRelationships = getDescendantsRequest.getIncludeRelationships();
            if(includeRelationships == null)
            {
                includeRelationships = IncludeRelationships.NONE;
            }
            Boolean includeAcls = getDescendantsRequest.getIncludeAcls();
            if(includeAcls == null)
            {
                includeAcls = Boolean.FALSE;
            }
            Set<String> propertyFilter = getDescendantsRequest.getPropertyFilter();
            Boolean includePolicies = getDescendantsRequest.getIncludePolicies();
            if(includePolicies != null)
            {
                includePolicies = Boolean.FALSE;
            }
            
	        if(folder != null && userId != null && networkId != null)
	        {
	            List<Tree<FileableCmisObject>> descendants = getPublicApi(userId).getDescendants(networkId, folder.getNodeId(), depth, includeRelationships, includeAcls, propertyFilter, includePolicies);
	            if(descendants != null)
	            {
	                folder.setDescendants(descendants);
	                response = new EventProcessorResponse("Got descendants of folder '" + folder, true, folder, true);
	            }
	            else
	            {
	                response = new EventProcessorResponse("Cannot get descendants of non-folder node '" + folder, true, folder, true);
	            }
	        }
	        else
	        {
	            response = new EventProcessorResponse("Failed to delete node, nodeId is null", false, null);
	        }
        }

        return response;
    }
}
