package org.alfresco.bm.publicapi.process;

import java.util.Map;

import org.alfresco.bm.Services;
import org.alfresco.bm.event.selector.EventProcessorResponse;
import org.alfresco.bm.event.selector.EventSelector;
import org.alfresco.bm.publicapi.AbstractRepositoryEventSelectorProcessor;
import org.alfresco.bm.publicapi.data.Request;
import org.alfresco.bm.publicapi.factory.PublicApiFactory;
import org.springframework.social.alfresco.api.entities.AlfrescoList;
import org.springframework.social.alfresco.api.entities.Site;

/**
 * Get favourite sites event processor.
 * 
 * @author steveglover
 *
 */
public class GetFavouriteSites extends AbstractRepositoryEventSelectorProcessor
{
    /**
     * @param services                access to collection services
     * @param publicApiFactory        a factory for creating connections to the public api
     * @param eventSelector           the event selector to use in selecting the next event
     */
    public GetFavouriteSites(Services services, PublicApiFactory publicApiFactory, EventSelector eventSelector)
    {
        super(services, publicApiFactory, eventSelector);
    }

    @Override
    public Object createDataObject(Object input, Object resp) throws Exception
    {
    	GetFavouriteSitesRequest ret = null;

        if(input != null)
        {
            Request request = (Request)input;
	        String networkId = request.getNetworkId();
	        String userId = request.getRunAsUserId();

	        // TODO randomize
	        int skipCount = 0;
	        int maxItems = 100;
		
    		ret = new GetFavouriteSitesRequest(networkId, userId, userId, skipCount, maxItems);
        }

        return ret;
    }

    @Override
    protected EventProcessorResponse processEventImpl(Object input) throws Exception
    {
        EventProcessorResponse response = null;

        if(input == null)
        {
        	 response = new EventProcessorResponse("Unable to get favourite sites, input is null", true, null);
        }
        else
        {
            GetFavouriteSitesRequest getFavouriteSitesRequest = (GetFavouriteSitesRequest)input;
            String userId = getFavouriteSitesRequest.getRunAsUserId();
            String networkId = getFavouriteSitesRequest.getNetworkId();
            String personId = getFavouriteSitesRequest.getPersonId();
            Integer skipCount = getFavouriteSitesRequest.getSkipCount();
            Integer maxItems = getFavouriteSitesRequest.getMaxItems();

	        if(personId != null && userId != null && networkId != null)
	        {
	            Map<String, String> parameters = new Parameters().addParameter("skipCount", skipCount).
	                    addParameter("maxItems", maxItems).toMap();
	
	            AlfrescoList<Site> favouriteSites = getPublicApi(userId).getFavoriteSites(networkId, personId, parameters);
	            response = new EventProcessorResponse("Got person favourite sites for user " + personId +
	                    ", run as " + userId + " in network " + networkId, true, favouriteSites, true);
	        }
	        else
	        {
	            response = new EventProcessorResponse("Cannot get person favourite sites, personId is null", false, null);
	        }
        }

        return response;
    }
}
