package org.alfresco.bm.publicapi.process;

import org.alfresco.bm.Services;
import org.alfresco.bm.event.selector.EventProcessorResponse;
import org.alfresco.bm.event.selector.EventSelector;
import org.alfresco.bm.publicapi.AbstractRepositoryEventSelectorProcessor;
import org.alfresco.bm.publicapi.data.Request;
import org.alfresco.bm.publicapi.factory.PublicApiFactory;
import org.springframework.social.alfresco.api.entities.Network;

/**
 * Get network details event processor.
 * 
 * @author steveglover
 *
 */
public class GetNetwork extends AbstractRepositoryEventSelectorProcessor
{
    /**
     * @param services                access to collection services
     * @param publicApiFactory        a factory for creating connections to the public api
     * @param eventSelector           the event selector to use in selecting the next event
     */
    public GetNetwork(Services services, PublicApiFactory publicApiFactory, EventSelector eventSelector)
    {
        super(services, publicApiFactory, eventSelector);
    }

    @Override
    public Object createDataObject(Object input, Object resp) throws Exception
    {
    	GetNetworkRequest ret = null;

        if(input != null)
        {
	        Request request = (Request)input;
	        String networkId = request.getNetworkId();
	        String userId = request.getRunAsUserId();
	        String whichNetworkId = randomNetwork();
	
	        ret = new GetNetworkRequest(networkId, userId, whichNetworkId);
        }
        
        return ret;
    }

    @Override
    protected EventProcessorResponse processEventImpl(Object input) throws Exception
    {
        EventProcessorResponse response = null;

        if(input == null)
        {
        	 response = new EventProcessorResponse("Unable to get network, input is null", true, null);
        }
        else
        {
            GetNetworkRequest getNetworkRequest = (GetNetworkRequest)input;
            String userId = getNetworkRequest.getRunAsUserId();
            String networkId = getNetworkRequest.getNetworkId();
            String whichNetworkId = getNetworkRequest.getWhichNetworkId();

	        if(whichNetworkId != null && userId != null && networkId != null)
	        {
	            Network network = getPublicApi(userId).getNetwork(whichNetworkId);
	            if(network != null)
	            {
	                response = new EventProcessorResponse("Got network " + whichNetworkId, true, network, true);
	            }
	            else
	            {
	                response = new EventProcessorResponse("Cannot get network " + whichNetworkId, true, null);
	            }
	        }
	        else
	        {
	            response = new EventProcessorResponse("Failed to get network, whichNetworkId is null", false, null);
	        }
        }

        return response;
    }
}
