package org.alfresco.bm.publicapi.process;

import org.alfresco.bm.Services;
import org.alfresco.bm.event.selector.EventProcessorResponse;
import org.alfresco.bm.event.selector.EventSelector;
import org.alfresco.bm.publicapi.AbstractRepositoryEventSelectorProcessor;
import org.alfresco.bm.publicapi.data.Node;
import org.alfresco.bm.publicapi.data.Request;
import org.alfresco.bm.publicapi.factory.PublicApiFactory;
import org.springframework.social.alfresco.api.entities.AlfrescoList;
import org.springframework.social.alfresco.api.entities.Rating;

/**
 * Get node ratings event processor.
 * 
 * @author steveglover
 *
 */
public class GetNodeRatings extends AbstractRepositoryEventSelectorProcessor
{
    /**
     * @param services                access to collection services
     * @param publicApiFactory        a factory for creating connections to the public api
     * @param eventSelector           the event selector to use in selecting the next event
     */
    public GetNodeRatings(Services services, PublicApiFactory publicApiFactory, EventSelector eventSelector)
    {
        super(services, publicApiFactory, eventSelector);
    }
    
    @Override
    public Object createDataObject(Object input, Object resp) throws Exception
    {
        GetNodeRatingsRequest ret = null;

        if(input != null)
        {
            Request request = (Request)input;

	        Node node = getNode(request, resp, currentNodeMatcher);
	        if(node != null)
	        {
	            logger.debug("GetNodeRatings.createDataObject node = " + node);
	
	            String networkId = request.getNetworkId();
	            String userId = request.getRunAsUserId();
	            // TODO more random...
	            int skipCount = 0;
	            int maxItems = Integer.MAX_VALUE;
	    
	            ret = new GetNodeRatingsRequest(networkId, userId, node, skipCount, maxItems);
	        }
        }
        
        return ret;
    }

    @Override
    protected EventProcessorResponse processEventImpl(Object input) throws Exception
    {
        EventProcessorResponse response = null;

        if(input == null)
        {
        	 response = new EventProcessorResponse("Unable to get node ratings, input is null", true, null);
        }
        else
        {
            GetNodeRatingsRequest getNodeRatingsRequest = (GetNodeRatingsRequest)input;
            String userId = getNodeRatingsRequest.getRunAsUserId();
            String networkId = getNodeRatingsRequest.getNetworkId();
            Node node = getNodeRatingsRequest.getNode();
            if(node != null && userId != null && networkId != null)
            {
	            AlfrescoList<Rating> nodeRatings = getPublicApi(userId).getNodeRatings(networkId, node.getNodeId());
	            node.setRatings(nodeRatings);
	            response = new EventProcessorResponse("Got ratings", true, node, true);
	        }
	        else
	        {
	            response = new EventProcessorResponse("Skipping get ratings: no nodeId supplied.", true, null);
	        }
        }

        return response;
    }
}
