package org.alfresco.bm.publicapi.process;

import org.alfresco.bm.Services;
import org.alfresco.bm.event.selector.EventProcessorResponse;
import org.alfresco.bm.event.selector.EventSelector;
import org.alfresco.bm.publicapi.AbstractRepositoryEventSelectorProcessor;
import org.alfresco.bm.publicapi.data.Node;
import org.alfresco.bm.publicapi.data.Request;
import org.alfresco.bm.publicapi.factory.PublicApiFactory;
import org.springframework.social.alfresco.api.entities.AlfrescoList;
import org.springframework.social.alfresco.api.entities.Tag;

/**
 * Get node tags event processor.
 * 
 * @author steveglover
 *
 */
public class GetNodeTags extends AbstractRepositoryEventSelectorProcessor
{
    /**
     * @param services                access to collection services
     * @param publicApiFactory        a factory for creating connections to the public api
     * @param eventSelector           the event selector to use in selecting the next event
     */
    public GetNodeTags(Services services, PublicApiFactory publicApiFactory, EventSelector eventSelector)
    {
        super(services, publicApiFactory, eventSelector);
    }

    @Override
    public Object createDataObject(Object input, Object resp) throws Exception
    {
        GetNodeTagsRequest ret = null;

        if(input != null)
        {
            Request request = (Request)input;

	        Node node = getNode(request, resp, currentNodeMatcher);
	        if(node != null)
	        {
	            logger.debug("GetNodeTags.createDataObject node = " + node);
	
	            String networkId = request.getNetworkId();
	            String userId = request.getRunAsUserId();
	            // TODO more random...
	            int skipCount = 0;
	            int maxItems = Integer.MAX_VALUE;
	            
	            ret = new GetNodeTagsRequest(networkId, userId, node, skipCount, maxItems);
	        }
        }
        
        return ret;
    }

    @Override
    protected EventProcessorResponse processEventImpl(Object input) throws Exception
    {
        EventProcessorResponse response = null;

        if(input == null)
        {
        	 response = new EventProcessorResponse("Unable to get node tags, input is null", true, null);
        }
        else
        {
            GetNodeTagsRequest getNodeTagsRequest = (GetNodeTagsRequest)input;
            String userId = getNodeTagsRequest.getRunAsUserId();
            String networkId = getNodeTagsRequest.getNetworkId();
            Node node = getNodeTagsRequest.getNode();

	        if(node != null && userId != null && networkId != null)
	        {
	            AlfrescoList<Tag> tags = getPublicApi(userId).getNodesTags(networkId, node.getNodeId());
	            node.setTags(tags);
	            response = new EventProcessorResponse("Got node tags for node " + node, true, node, true);
	        }
	        else
	        {
	            response = new EventProcessorResponse("Cannot get node tags, nodeId is null", false, null);
	        }
        }

        return response;
    }
}
