package org.alfresco.bm.publicapi.process;

import org.alfresco.bm.Services;
import org.alfresco.bm.event.selector.EventProcessorResponse;
import org.alfresco.bm.event.selector.EventSelector;
import org.alfresco.bm.publicapi.AbstractRepositoryEventSelectorProcessor;
import org.alfresco.bm.publicapi.data.Node;
import org.alfresco.bm.publicapi.data.Request;
import org.alfresco.bm.publicapi.factory.PublicApiFactory;
import org.apache.chemistry.opencmis.client.api.CmisObject;
import org.apache.chemistry.opencmis.client.api.Session;

/**
 * Get object by id.
 * 
 * @author steveglover
 *
 */
public class GetObject extends AbstractRepositoryEventSelectorProcessor
{
    /**
     * @param services                access to collection services
     * @param publicApiFactory        a factory for creating connections to the public api
     * @param eventSelector           the event selector to use in selecting the next event
     */
    public GetObject(Services services, PublicApiFactory publicApiFactory, EventSelector eventSelector)
    {
        super(services, publicApiFactory, eventSelector);
    }

    @Override
    public Object createDataObject(Object input, Object resp) throws Exception
    {
        GetObjectRequest ret = null;

        if(input != null)
        {
            Request request = (Request)input;

	        // assume that we can get the node, given that we have the object in a response
	        Node node = getNode(request, resp, matchAllNodes);
	        if(node != null)
	        {
	        	String objectId = node.getNodeId();
	            logger.debug("GetObject.createDataObject objectId = " + objectId);
	
	            String networkId = request.getNetworkId();
	            String userId = request.getRunAsUserId();
	            ret = new GetObjectRequest(networkId, userId, objectId);
	        }
        }
        
        return ret;
    }

    @Override
    protected EventProcessorResponse processEventImpl(Object input) throws Exception
    {
        EventProcessorResponse response = null;

        if(input == null)
        {
        	response = new EventProcessorResponse("Unable to get object, input is null", true, null);
        }
        else
        {
        	GetObjectRequest getObjectRequest = (GetObjectRequest)input;
            String userId = getObjectRequest.getRunAsUserId();
            String networkId = getObjectRequest.getNetworkId();
            String objectId = getObjectRequest.getObjectId();
            if(objectId != null && userId != null && networkId != null)
            {
	            Session cmisSession = getPublicApi(userId).getCMISSession(networkId);
	            CmisObject obj = cmisSession.getObject(objectId);
	            Node node = Node.createNode(obj);
	
	            response = new EventProcessorResponse("Got object " + objectId + " = " + node, true, node, true);
	        }
	        else
	        {
	            response = new EventProcessorResponse("Cannot get object", false, null);
	        }
        }

        return response;
    }

}
