package org.alfresco.bm.publicapi.process;

import org.alfresco.bm.Services;
import org.alfresco.bm.event.selector.EventProcessorResponse;
import org.alfresco.bm.event.selector.EventSelector;
import org.alfresco.bm.publicapi.AbstractRepositoryEventSelectorProcessor;
import org.alfresco.bm.publicapi.data.Node;
import org.alfresco.bm.publicapi.data.Request;
import org.alfresco.bm.publicapi.factory.PublicApiFactory;
import org.apache.chemistry.opencmis.client.api.CmisObject;
import org.apache.chemistry.opencmis.client.api.Session;

/**
 * Get an object by path.
 * 
 * @author steveglover
 *
 */
public class GetObjectByPath extends AbstractRepositoryEventSelectorProcessor
{
    /**
     * @param services                access to collection services
     * @param publicApiFactory        a factory for creating connections to the public api
     * @param eventSelector           the event selector to use in selecting the next event
     */
    public GetObjectByPath(Services services, PublicApiFactory publicApiFactory, EventSelector eventSelector)
    {
        super(services, publicApiFactory, eventSelector);
    }

    @Override
    public Object createDataObject(Object input, Object response) throws Exception
    {
        GetObjectByPathRequest ret = null;

        if(input != null)
        {
            Request request = (Request)input;

	        String path = getPath(request, response);
	        if(path != null)
	        {
	            logger.debug("GetObjectByPath.createDataObject path = " + path);
	
	            String networkId = request.getNetworkId();
	            String userId = request.getRunAsUserId();
	            ret = new GetObjectByPathRequest(networkId, userId, path);
	        }
        }
        
        return ret;
    }

    @Override
    protected EventProcessorResponse processEventImpl(Object input) throws Exception
    {
        EventProcessorResponse response = null;

        if(input == null)
        {
        	 response = new EventProcessorResponse("Unable to get object by path, input is null", true, null);
        }
        else
        {
        	GetObjectByPathRequest getObjectByPathRequest = (GetObjectByPathRequest)input;
            String userId = getObjectByPathRequest.getRunAsUserId();
            String networkId = getObjectByPathRequest.getNetworkId();
            String path = getObjectByPathRequest.getPath();
            
            if(path != null && userId != null && networkId != null)
            {
	            Session cmisSession = getPublicApi(userId).getCMISSession(networkId);
	            CmisObject obj = cmisSession.getObjectByPath(path);
	            Node node = Node.createNode(obj);
	
	            response = new EventProcessorResponse("Got object by path " + path + " = " + node, true, node, true);
	        }
	        else
	        {
	            response = new EventProcessorResponse("Cannot get object by path", false, null);
	        }
        }

        return response;
    }

}
