package org.alfresco.bm.publicapi.process;

import org.alfresco.bm.Services;
import org.alfresco.bm.event.selector.EventProcessorResponse;
import org.alfresco.bm.event.selector.EventSelector;
import org.alfresco.bm.publicapi.AbstractRepositoryEventSelectorProcessor;
import org.alfresco.bm.publicapi.data.Request;
import org.alfresco.bm.publicapi.factory.PublicApiFactory;
import org.alfresco.bm.user.UserData;
import org.springframework.social.alfresco.api.entities.Person;

/**
 * Get person event processor.
 * 
 * @author steveglover
 *
 */
public class GetPerson extends AbstractRepositoryEventSelectorProcessor
{
    /**
     * @param services                access to collection services
     * @param publicApiFactory        a factory for creating connections to the public api
     * @param eventSelector           the event selector to use in selecting the next event
     */
    public GetPerson(Services services, PublicApiFactory publicApiFactory, EventSelector eventSelector)
    {
        super(services, publicApiFactory, eventSelector);
    }
    
    @Override
    public Object createDataObject(Object input, Object response) throws Exception
    {
    	GetPersonRequest ret = null;
    	
        if(input != null)
        {
	        Request request = (Request)input;
	        UserData person = services.getUserDataService().randomUser();
	        if(person != null)
	        {
		        String networkId = request.getNetworkId();
		        String userId = request.getRunAsUserId();
		
		        ret = new GetPersonRequest(networkId, userId, person.getUsername());
	        }
        }
        
        return ret;
    }

    @Override
    protected EventProcessorResponse processEventImpl(Object input) throws Exception
    {
        EventProcessorResponse response = null;

        if(input == null)
        {
        	 response = new EventProcessorResponse("Unable to get person, input is null", true, null);
        }
        else
        {
            GetPersonRequest getPersonRequest = (GetPersonRequest)input;
            String userId = getPersonRequest.getRunAsUserId();
            String networkId = getPersonRequest.getNetworkId();
            String personId = getPersonRequest.getPersonId();

	        if(personId != null && userId != null && networkId != null)
	        {
	            Person person = getPublicApi(userId).getPerson(networkId, personId);
	            response = new EventProcessorResponse("Got person", true, person, true);
	        }
	        else
	        {
	            response = new EventProcessorResponse("Skipping get person: no personId supplied.", true, null);
	        }
        }

        return response;
    }
}
