package org.alfresco.bm.publicapi.process;

import java.util.HashMap;
import java.util.Map;

import org.alfresco.bm.Services;
import org.alfresco.bm.event.selector.EventProcessorResponse;
import org.alfresco.bm.event.selector.EventSelector;
import org.alfresco.bm.publicapi.AbstractRepositoryEventSelectorProcessor;
import org.alfresco.bm.publicapi.data.Request;
import org.alfresco.bm.publicapi.factory.PublicApiFactory;
import org.springframework.social.alfresco.api.entities.Activity;
import org.springframework.social.alfresco.api.entities.Activity.Who;
import org.springframework.social.alfresco.api.entities.AlfrescoList;

/**
 * Get person activities event processor.
 * 
 * @author steveglover
 *
 */
public class GetPersonActivities extends AbstractRepositoryEventSelectorProcessor
{
    /**
     * @param services                access to collection services
     * @param publicApiFactory        a factory for creating connections to the public api
     * @param eventSelector           the event selector to use in selecting the next event
     */
    public GetPersonActivities(Services services, PublicApiFactory publicApiFactory, EventSelector eventSelector)
    {
        super(services, publicApiFactory, eventSelector);
    }
    
    @Override
    public Object createDataObject(Object input, Object response) throws Exception
    {
    	GetPersonActivitiesRequest ret = null;
    	
        if(input != null && response != null)
        {
	        Request request = (Request)input;
	        String networkId = request.getNetworkId();
	        String userId = request.getRunAsUserId();

	        Who who = randomWho();
	        boolean nullSite = random.nextInt(2) == 0 ? false : true;
	        String siteId = nullSite ? null : randomSite(networkId);
	        // TODO more random...
	        int skipCount = 0;
	        int maxItems = Integer.MAX_VALUE;
	        
	        ret = new GetPersonActivitiesRequest(networkId, userId, userId, who, siteId, skipCount, maxItems);
        }

        return ret;
    }

    @Override
    protected EventProcessorResponse processEventImpl(Object input) throws Exception
    {
        EventProcessorResponse response = null;

        if(input == null)
        {
        	 response = new EventProcessorResponse("Unable to get person activities, input is null", true, null);
        }
        else
        {
            GetPersonActivitiesRequest getPersonActivitiesRequest = (GetPersonActivitiesRequest)input;
            String userId = getPersonActivitiesRequest.getRunAsUserId();
            String networkId = getPersonActivitiesRequest.getNetworkId();
            String personId = getPersonActivitiesRequest.getPersonId();
            Who who = getPersonActivitiesRequest.getWho();
            String siteId = getPersonActivitiesRequest.getSiteId();

	        if(personId != null && userId != null && networkId != null)
	        {
	            Map<String, String> parameters = new HashMap<String, String>();
	            if(who != null)
	            {
	                parameters.put(Activity.WHO, who.toString());
	            }
	            if(siteId != null)
	            {
	                parameters.put(Activity.SITEID, siteId);
	            }
	
	            AlfrescoList<Activity> activities = getPublicApi(userId).getActivities(networkId, personId, parameters);
	            response = new EventProcessorResponse("Got person activities for user " + personId, true, activities, true);
	        }
	        else
	        {
	            response = new EventProcessorResponse("Cannot get person activities, personId is null", false, null);
	        }
        }

        return response;
    }
}
