package org.alfresco.bm.publicapi.process;

import org.alfresco.bm.Services;
import org.alfresco.bm.event.selector.EventProcessorResponse;
import org.alfresco.bm.event.selector.EventSelector;
import org.alfresco.bm.publicapi.AbstractRepositoryEventSelectorProcessor;
import org.alfresco.bm.publicapi.data.Request;
import org.alfresco.bm.publicapi.factory.PublicApiFactory;
import org.springframework.social.alfresco.api.entities.AlfrescoList;
import org.springframework.social.alfresco.api.entities.Network;

/**
 * Get person networks event processor.
 * 
 * @author steveglover
 *
 */
public class GetPersonNetworks extends AbstractRepositoryEventSelectorProcessor
{
    /**
     * @param services                access to collection services
     * @param publicApiFactory        a factory for creating connections to the public api
     * @param eventSelector           the event selector to use in selecting the next event
     */
    public GetPersonNetworks(Services services, PublicApiFactory publicApiFactory, EventSelector eventSelector)
    {
        super(services, publicApiFactory, eventSelector);
    }
    
    @Override
    public Object createDataObject(Object input, Object response) throws Exception
    {
    	GetPersonNetworksRequest ret = null;
    	
        if(input != null)
        {
	        Request request = (Request)input;
	        String networkId = request.getNetworkId();
	        String userId = request.getRunAsUserId();
	
	        ret = new GetPersonNetworksRequest(networkId, userId, userId);
        }
        
        return ret;
    }

    @Override
    protected EventProcessorResponse processEventImpl(Object input) throws Exception
    {
        EventProcessorResponse response = null;

        if(input == null)
        {
        	 response = new EventProcessorResponse("Unable to get person networks, input is null", true, null);
        }
        else
        {
            GetPersonNetworksRequest getPersonNetworksRequest = (GetPersonNetworksRequest)input;
            String userId = getPersonNetworksRequest.getRunAsUserId();
            String networkId = getPersonNetworksRequest.getNetworkId();
            String personId = getPersonNetworksRequest.getPersonId();
            
	        if(personId != null && userId != null && networkId != null)
	        {
	            AlfrescoList<Network> networks = getPublicApi(userId).getNetworks(networkId, personId);
	            response = new EventProcessorResponse("Got person networks", true, networks, true);
	        }
	        else
	        {
	            response = new EventProcessorResponse("Skipping get person networks: no personId supplied.", true, null);
	        }
        }
        
        return response;
    }
}
