package org.alfresco.bm.publicapi.process;

import org.alfresco.bm.Services;
import org.alfresco.bm.event.selector.EventProcessorResponse;
import org.alfresco.bm.event.selector.EventSelector;
import org.alfresco.bm.publicapi.AbstractRepositoryEventSelectorProcessor;
import org.alfresco.bm.publicapi.data.Request;
import org.alfresco.bm.publicapi.factory.PublicApiFactory;
import org.springframework.social.alfresco.api.entities.AlfrescoList;
import org.springframework.social.alfresco.api.entities.Preference;

/**
 * Get person preferences event processor.
 * 
 * @author steveglover
 *
 */
public class GetPersonPreferences extends AbstractRepositoryEventSelectorProcessor
{
    /**
     * @param services                access to collection services
     * @param publicApiFactory        a factory for creating connections to the public api
     * @param eventSelector           the event selector to use in selecting the next event
     */
    public GetPersonPreferences(Services services, PublicApiFactory publicApiFactory, EventSelector eventSelector)
    {
        super(services, publicApiFactory, eventSelector);
    }
    
    @Override
    public Object createDataObject(Object input, Object response) throws Exception
    {
    	GetPersonPreferencesRequest ret = null;
    	
        if(input != null)
        {
	        Request request = (Request)input;
	        String networkId = request.getNetworkId();
	        String userId = request.getRunAsUserId();

	        ret = new GetPersonPreferencesRequest(networkId, userId, userId);
        }
        
        return ret;
    }

    @Override
    protected EventProcessorResponse processEventImpl(Object input) throws Exception
    {
        EventProcessorResponse response = null;

        if(input == null)
        {
        	 response = new EventProcessorResponse("Unable to get person preferences, input is null", true, null);
        }
        else
        {
            GetPersonPreferencesRequest getPersonPreferencesRequest = (GetPersonPreferencesRequest)input;
            String userId = getPersonPreferencesRequest.getRunAsUserId();
            String networkId = getPersonPreferencesRequest.getNetworkId();
            String personId = getPersonPreferencesRequest.getPersonId();

	        if(userId != null && networkId != null && personId != null)
	        {
	            AlfrescoList<Preference> preferences = getPublicApi(userId).getPreferences(networkId, personId);
	            response = new EventProcessorResponse("Got person preferences", true, preferences, true);
	        }
	        else
	        {
	            response = new EventProcessorResponse("Skipping get person preferences: no personId supplied.", true, null);
	        }
        }

        return response;
    }
}
