package org.alfresco.bm.publicapi.process;

import java.util.Map;

import org.alfresco.bm.Services;
import org.alfresco.bm.event.selector.EventProcessorResponse;
import org.alfresco.bm.event.selector.EventSelector;
import org.alfresco.bm.publicapi.AbstractRepositoryEventSelectorProcessor;
import org.alfresco.bm.publicapi.data.Request;
import org.alfresco.bm.publicapi.factory.PublicApiFactory;
import org.springframework.social.alfresco.api.entities.AlfrescoList;
import org.springframework.social.alfresco.api.entities.Site;

/**
 * Get person sites event processor.
 * 
 * @author steveglover
 *
 */
public class GetPersonSites extends AbstractRepositoryEventSelectorProcessor
{
    /**
     * @param services                access to collection services
     * @param publicApiFactory        a factory for creating connections to the public api
     * @param eventSelector           the event selector to use in selecting the next event
     */
    public GetPersonSites(Services services, PublicApiFactory publicApiFactory, EventSelector eventSelector)
    {
        super(services, publicApiFactory, eventSelector);
    }
    
    @Override
    public Object createDataObject(Object input, Object response) throws Exception
    {
    	GetPersonSitesRequest ret = null;
    	
        if(input != null)
        {
	        Request request = (Request)input;
	        String networkId = request.getNetworkId();
	        String userId = request.getRunAsUserId();
	        Integer skipCount = Integer.valueOf(0);
	        Integer maxItems = Integer.valueOf(40);
	        ret = new GetPersonSitesRequest(networkId, userId, userId, skipCount, maxItems);
        }

        return ret;
    }

    @Override
    protected EventProcessorResponse processEventImpl(Object input) throws Exception
    {
        EventProcessorResponse response = null;

        if(input == null)
        {
        	 response = new EventProcessorResponse("Unable to get person sites, input is null", true, null);
        }
        else
        {
            GetPersonSitesRequest getPersonSitesRequest = (GetPersonSitesRequest)input;
            String userId = getPersonSitesRequest.getRunAsUserId();
            String networkId = getPersonSitesRequest.getNetworkId();
            String personId = getPersonSitesRequest.getPersonId();
            Integer skipCount = getPersonSitesRequest.getSkipCount();
            Integer maxItems = getPersonSitesRequest.getMaxItems();

	        if(userId != null && networkId != null && personId != null)
	        {
	            Map<String, String> parameters = new Parameters().addParameter("skipCount", skipCount).
	                    addParameter("maxItems", maxItems).toMap();

	            AlfrescoList<Site> personSites = getPublicApi(userId).getSites(networkId, personId, parameters);
	            response = new EventProcessorResponse("Got person sites", true, personSites, true);
	        }
	        else
	        {
	            response = new EventProcessorResponse("Skipping get person sites: invalid request: " + getPersonSitesRequest, true, null);
	        }
        }

        return response;
    }
}
