package org.alfresco.bm.publicapi.process;

import org.alfresco.bm.Services;
import org.alfresco.bm.event.selector.EventProcessorResponse;
import org.alfresco.bm.event.selector.EventSelector;
import org.alfresco.bm.publicapi.AbstractRepositoryEventSelectorProcessor;
import org.alfresco.bm.publicapi.data.Node;
import org.alfresco.bm.publicapi.data.Request;
import org.alfresco.bm.publicapi.factory.PublicApiFactory;
import org.apache.chemistry.opencmis.client.api.CmisObject;
import org.apache.chemistry.opencmis.client.api.Session;

/**
 * Get object properties.
 * 
 * @author steveglover
 *
 */
public class GetProperties extends AbstractRepositoryEventSelectorProcessor
{
    /**
     * @param services                access to collection services
     * @param publicApiFactory        a factory for creating connections to the public api
     * @param eventSelector           the event selector to use in selecting the next event
     */
    public GetProperties(Services services, PublicApiFactory publicApiFactory, EventSelector eventSelector)
    {
        super(services, publicApiFactory, eventSelector);
    }

    @Override
    public Object createDataObject(Object input, Object response) throws Exception
    {
        GetPropertiesRequest ret = null;
        
        if(input != null)
        {
            Request request = (Request)input;

	        Node node = getNode(request, response, canGetPropertiesNodeMatcher);
	        if(node != null)
	        {
	            logger.debug("GetProperties.createDataObject node = " + node);
	
	            String networkId = request.getNetworkId();
	            String userId = request.getRunAsUserId();
	            ret = new GetPropertiesRequest(networkId, userId, node);
	        }
        }
        
        return ret;
    }

    @Override
    protected EventProcessorResponse processEventImpl(Object input) throws Exception
    {
        EventProcessorResponse response = null;

        if(input == null)
        {
        	 response = new EventProcessorResponse("Unable to get properties, input is null", true, null);
        }
        else
        {
        	GetPropertiesRequest getPropertiesRequest = (GetPropertiesRequest)input;
            String userId = getPropertiesRequest.getRunAsUserId();
            String networkId = getPropertiesRequest.getNetworkId();
            Node node = getPropertiesRequest.getNode();

	        if(node != null && userId != null && networkId != null)
	        {
	            Session cmisSession = getPublicApi(userId).getCMISSession(networkId);
	            CmisObject obj = cmisSession.getObject(node.getNodeId());
	            node.setProperties(obj.getProperties());
	
	            response = new EventProcessorResponse("Got object properties " + node, true, node, true);
	        }
	        else
	        {
	            response = new EventProcessorResponse("Cannot get object properties", false, node);
	        }
        }

        return response;
    }
}
