package org.alfresco.bm.publicapi.process;

import org.alfresco.bm.Services;
import org.alfresco.bm.event.selector.EventProcessorResponse;
import org.alfresco.bm.event.selector.EventSelector;
import org.alfresco.bm.publicapi.AbstractRepositoryEventSelectorProcessor;
import org.alfresco.bm.publicapi.data.Request;
import org.alfresco.bm.publicapi.factory.PublicApiFactory;
import org.springframework.social.alfresco.api.entities.Site;

/**
 * Get site event processor.
 * 
 * @author steveglover
 *
 */
public class GetSite extends AbstractRepositoryEventSelectorProcessor
{
    /**
     * @param services                access to collection services
     * @param publicApiFactory        a factory for creating connections to the public api
     * @param eventSelector           the event selector to use in selecting the next event
     */
    public GetSite(Services services, PublicApiFactory publicApiFactory, EventSelector eventSelector)
    {
        super(services, publicApiFactory, eventSelector);
    }
    
    @Override
    public Object createDataObject(Object input, Object response) throws Exception
    {
    	GetSiteRequest ret = null;

        if(input != null)
        {
	        Request request = (Request)input;
	        String networkId = request.getNetworkId();
	        String userId = request.getRunAsUserId();
	        String siteId = randomSite(networkId);
	        ret = new GetSiteRequest(networkId, userId, siteId);
        }
        
        return ret;
    }

    @Override
    protected EventProcessorResponse processEventImpl(Object input) throws Exception
    {
        EventProcessorResponse response = null;

        if(input == null)
        {
        	 response = new EventProcessorResponse("Unable to get site, input is null", true, null);
        }
        else
        {
            GetSiteRequest getSiteRequest = (GetSiteRequest)input;
            String userId = getSiteRequest.getRunAsUserId();
            String networkId = getSiteRequest.getNetworkId();
            String siteId = getSiteRequest.getSiteId();

	        if(userId != null && networkId != null && siteId != null)
	        {
	            Site site = getPublicApi(userId).getSite(siteId, networkId);
	            response = new EventProcessorResponse("Got site", true, site, true);
	        }
	        else
	        {
	            response = new EventProcessorResponse("Skipping get site : invalid request: " + getSiteRequest, true, null);
	        }
        }

        return response;
    }

}
