package org.alfresco.bm.publicapi.process;

import org.alfresco.bm.Services;
import org.alfresco.bm.event.selector.EventProcessorResponse;
import org.alfresco.bm.event.selector.EventSelector;
import org.alfresco.bm.publicapi.AbstractRepositoryEventSelectorProcessor;
import org.alfresco.bm.publicapi.data.Request;
import org.alfresco.bm.publicapi.factory.PublicApiFactory;
import org.springframework.social.alfresco.api.entities.AlfrescoList;
import org.springframework.social.alfresco.api.entities.Container;
import org.springframework.social.alfresco.api.entities.Site;

/**
 * Get site containers event processor.
 * 
 * @author steveglover
 *
 */
public class GetSiteContainers extends AbstractRepositoryEventSelectorProcessor
{
    /**
     * @param services                access to collection services
     * @param publicApiFactory        a factory for creating connections to the public api
     * @param eventSelector           the event selector to use in selecting the next event
     */
    public GetSiteContainers(Services services, PublicApiFactory publicApiFactory, EventSelector eventSelector)
    {
        super(services, publicApiFactory, eventSelector);
    }

    @Override
    public Object createDataObject(Object input, Object response) throws Exception
    {
    	GetSiteContainersRequest ret = null;
    	
        if(input != null)
        {
	        Request request = (Request)input;
	    	Site site = getSite(request, response);
	    	if(site != null)
	    	{
		        String networkId = request.getNetworkId();
		        String userId = request.getRunAsUserId();
		        ret = new GetSiteContainersRequest(networkId, userId, site);
	    	}
        }

        return ret;
    }

    @Override
    protected EventProcessorResponse processEventImpl(Object input) throws Exception
    {
        EventProcessorResponse response = null;

        if(input == null)
        {
        	 response = new EventProcessorResponse("Unable to get site containers, input is null", true, null);
        }
        else
        {
            GetSiteContainersRequest getSiteContainersRequest = (GetSiteContainersRequest)input;
            String userId = getSiteContainersRequest.getRunAsUserId();
            String networkId = getSiteContainersRequest.getNetworkId();
            Site site = getSiteContainersRequest.getSite();

	        if(userId == null)
	        {
	            response = new EventProcessorResponse("Skipping get site containers: no userId supplied.", true, null);
	        }
	        else if(networkId == null)
	        {
	            response = new EventProcessorResponse("Skipping get site containers: no networkId supplied.", true, null);
	        }
	        else if(site == null)
	        {
	            response = new EventProcessorResponse("Skipping get site containers: no Site supplied.", true, null);
	        }
	        else
	        {
	            AlfrescoList<Container> siteContainers = getPublicApi(userId).getContainers(networkId, site.getId());
	            site.setContainers(siteContainers);
	            response = new EventProcessorResponse("Got site containers for site " + site, true, site, true);
	        }
        }

        return response;
    }
}
