package org.alfresco.bm.publicapi.process;

import org.alfresco.bm.Services;
import org.alfresco.bm.event.selector.EventProcessorResponse;
import org.alfresco.bm.event.selector.EventSelector;
import org.alfresco.bm.publicapi.AbstractRepositoryEventSelectorProcessor;
import org.alfresco.bm.publicapi.data.Request;
import org.alfresco.bm.publicapi.factory.PublicApiFactory;
import org.springframework.social.alfresco.api.entities.AlfrescoList;
import org.springframework.social.alfresco.api.entities.Member;

/**
 * Get site members event processor.
 * 
 * @author steveglover
 *
 */
public class GetSiteMembers extends AbstractRepositoryEventSelectorProcessor
{
    /**
     * @param services                access to collection services
     * @param publicApiFactory        a factory for creating connections to the public api
     * @param eventSelector           the event selector to use in selecting the next event
     */
    public GetSiteMembers(Services services, PublicApiFactory publicApiFactory, EventSelector eventSelector)
    {
        super(services, publicApiFactory, eventSelector);
    }


    @Override
    public Object createDataObject(Object input, Object response) throws Exception
    {
    	GetSiteMembersRequest ret = null;

        if(input != null)
        {
	        Request request = (Request)input;
	        String networkId = request.getNetworkId();
	        String userId = request.getRunAsUserId();
	        
	        String siteId = randomSiteForMember(userId, networkId);
	
	        // TODO more random...
	        int skipCount = 0;
	        int maxItems = Integer.MAX_VALUE;
	
	        ret = new GetSiteMembersRequest(networkId, userId, siteId, skipCount, maxItems);
        }
        
        return ret;
    }

    @Override
    protected EventProcessorResponse processEventImpl(Object input) throws Exception
    {
        EventProcessorResponse response = null;

        if(input == null)
        {
        	 response = new EventProcessorResponse("Unable to get site members, input is null", true, null);
        }
        else
        {
            GetSiteMembersRequest getSiteMembersRequest = (GetSiteMembersRequest)input;
            String userId = getSiteMembersRequest.getRunAsUserId();
            String networkId = getSiteMembersRequest.getNetworkId();
            String siteId = getSiteMembersRequest.getSiteId();
	
	        if(siteId != null && userId != null && networkId != null)
	        {
	            AlfrescoList<Member> siteMembers = getPublicApi(userId).getMembers(networkId, siteId);
	            response = new EventProcessorResponse("Got site members", true, siteMembers, true);
	        }
	        else
	        {
	            response = new EventProcessorResponse("Skipping get site members: no siteId supplied.", true, null);
	        }
        }

        return response;
    }
}
