package org.alfresco.bm.publicapi.process;

import java.util.Map;

import org.alfresco.bm.Services;
import org.alfresco.bm.event.selector.EventProcessorResponse;
import org.alfresco.bm.event.selector.EventSelector;
import org.alfresco.bm.publicapi.AbstractRepositoryEventSelectorProcessor;
import org.alfresco.bm.publicapi.data.Request;
import org.alfresco.bm.publicapi.factory.PublicApiFactory;
import org.springframework.social.alfresco.api.entities.AlfrescoList;
import org.springframework.social.alfresco.api.entities.Site;

/**
 * Get sites event processor.
 * 
 * @author steveglover
 *
 */
public class GetSites extends AbstractRepositoryEventSelectorProcessor
{
    /**
     * @param services                access to collection services
     * @param publicApiFactory        a factory for creating connections to the public api
     * @param eventSelector           the event selector to use in selecting the next event
     */
    public GetSites(Services services, PublicApiFactory publicApiFactory, EventSelector eventSelector)
    {
        super(services, publicApiFactory, eventSelector);
    }
    
    @Override
    public Object createDataObject(Object input, Object response) throws Exception
    {
    	GetSitesRequest ret = null;

        if(input != null)
        {
	        Request request = (Request)input;
	        String networkId = request.getNetworkId();
	        String userId = request.getRunAsUserId();
	        // TODO more random...
	        int skipCount = 0;
	        int maxItems = 40;

	        ret = new GetSitesRequest(networkId, userId, skipCount, maxItems);
        }
        
        return ret;
    }

    @Override
    protected EventProcessorResponse processEventImpl(Object input) throws Exception
    {
        EventProcessorResponse response = null;
        
        if(input == null)
        {
        	 response = new EventProcessorResponse("Unable to get sites, input is null", true, null);
        }
        else
        {
            GetSitesRequest getSitesRequest = (GetSitesRequest)input;
	        String userId = getSitesRequest.getRunAsUserId();
	        String networkId = getSitesRequest.getNetworkId();
	        Integer skipCount = getSitesRequest.getSkipCount();
	        Integer maxItems = getSitesRequest.getMaxItems();
            Map<String, String> parameters = new Parameters().addParameter("skipCount", skipCount).
                    addParameter("maxItems", maxItems).toMap();
	
	        if (userId != null && networkId != null)
	        {
	            AlfrescoList<Site> sites = getPublicApi(userId).getSites(networkId, parameters);
	            response = new EventProcessorResponse("Got " + sites.getEntries().size() + " sites: " + getSitesRequest, true, sites, true);
	        }
	        else
	        {
	            response = new EventProcessorResponse("Skipping get sites : invalid request: " + getSitesRequest, true, null);
	        }
        }

        return response;
    }
}
