package org.alfresco.bm.publicapi.process;

import org.alfresco.bm.Services;
import org.alfresco.bm.event.selector.EventProcessorResponse;
import org.alfresco.bm.event.selector.EventSelector;
import org.alfresco.bm.publicapi.AbstractRepositoryEventSelectorProcessor;
import org.alfresco.bm.publicapi.data.Request;
import org.alfresco.bm.publicapi.factory.PublicApiFactory;
import org.springframework.social.alfresco.api.entities.AlfrescoList;
import org.springframework.social.alfresco.api.entities.Tag;

/**
 * Get network tags event processor.
 * 
 * @author steveglover
 *
 */
public class GetTags extends AbstractRepositoryEventSelectorProcessor
{
    /**
     * @param services                access to collection services
     * @param publicApiFactory        a factory for creating connections to the public api
     * @param eventSelector           the event selector to use in selecting the next event
     */
    public GetTags(Services services, PublicApiFactory publicApiFactory, EventSelector eventSelector)
    {
        super(services, publicApiFactory, eventSelector);
    }

    @Override
    public Object createDataObject(Object input, Object response) throws Exception
    {
    	GetTagsRequest ret = null;

        if(input != null)
        {
	        Request request = (Request)input;
	        String networkId = request.getNetworkId();
	        String userId = request.getRunAsUserId();
	        // TODO more random...
	        int skipCount = 0;
	        int maxItems = Integer.MAX_VALUE;
	        
	        ret = new GetTagsRequest(networkId, userId, skipCount, maxItems);
    	}

        return ret;
    }

    @Override
    protected EventProcessorResponse processEventImpl(Object input) throws Exception
    {
        EventProcessorResponse response = null;

        if(input == null)
        {
        	 response = new EventProcessorResponse("Unable to get tags, input is null", true, null);
        }
        else
        {
	        GetTagsRequest getTagsRequest = (GetTagsRequest)input;
	        String userId = getTagsRequest.getRunAsUserId();
	        String networkId = getTagsRequest.getNetworkId();
	
	        if(userId != null && networkId != null)
	        {
	            AlfrescoList<Tag> tags = getPublicApi(userId).getTags(networkId);
	            response = new EventProcessorResponse("Got tags", true, tags, true);
	        }
	        else
	        {
	            response = new EventProcessorResponse("Skipping get tags : invalid request: " + getTagsRequest, true, null);
	        }
        }

        return response;
    }    
}
