package org.alfresco.bm.publicapi.process;

import org.alfresco.bm.Services;
import org.alfresco.bm.event.selector.EventProcessorResponse;
import org.alfresco.bm.event.selector.EventSelector;
import org.alfresco.bm.publicapi.AbstractRepositoryEventSelectorProcessor;
import org.alfresco.bm.publicapi.data.Node;
import org.alfresco.bm.publicapi.data.Request;
import org.alfresco.bm.publicapi.factory.PublicApiFactory;
import org.springframework.social.alfresco.api.entities.Rating;

/**
 * Remove node rating event processor.
 * 
 * @author steveglover
 *
 */
public class RemoveNodeRating extends AbstractRepositoryEventSelectorProcessor
{
    /**
     * @param services                access to collection services
     * @param publicApiFactory        a factory for creating connections to the public api
     * @param eventSelector           the event selector to use in selecting the next event
     */
    public RemoveNodeRating(Services services, PublicApiFactory publicApiFactory, EventSelector eventSelector)
    {
        super(services, publicApiFactory, eventSelector);
    }
    
    @Override
    public Object createDataObject(Object input, Object response) throws Exception
    {
        RemoveNodeRatingRequest ret = null;

        if(input != null)
        {
            Request request = (Request)input;

	        Node node = getNode(request, response, matchAllNodes); // TODO need to check that the user can remove the rating
	        if(node != null)
	        {
	            logger.debug("RemoveNodeRating.createDataObject node = " + node);
	
	            String ratingId = Rating.LIKES;
	            String networkId = request.getNetworkId();
	            String userId = request.getRunAsUserId();
	    
	            ret = new RemoveNodeRatingRequest(networkId, userId, node, ratingId);
	        }
        }
        
        return ret;
    }

    @Override
    protected EventProcessorResponse processEventImpl(Object input) throws Exception
    {
        EventProcessorResponse response = null;

        if(input == null)
        {
        	 response = new EventProcessorResponse("Skipping remove node rating, input is null", true, null);
        }
        else
        {
            RemoveNodeRatingRequest removeNodeRatingRequest = (RemoveNodeRatingRequest)input;
            String userId = removeNodeRatingRequest.getRunAsUserId();
            String networkId = removeNodeRatingRequest.getNetworkId();
            Node node = removeNodeRatingRequest.getNode();
            String ratingId = removeNodeRatingRequest.getRatingId();

	        if(node != null && ratingId != null && userId != null && networkId != null)
	        {
	            getPublicApi(userId).removeNodeRating(networkId, node.getNodeId(), ratingId);
	            response = new EventProcessorResponse("Removed node rating", true, node);
	        }
	        else
	        {
	            response = new EventProcessorResponse("Skipping remove node rating: no nodeId or ratingId supplied.", true, null);
	        }
        }

        return response;
    }
}
