package org.alfresco.bm.publicapi.process;

import org.alfresco.bm.Services;
import org.alfresco.bm.event.selector.EventProcessorResponse;
import org.alfresco.bm.event.selector.EventSelector;
import org.alfresco.bm.publicapi.AbstractRepositoryEventSelectorProcessor;
import org.alfresco.bm.publicapi.data.Request;
import org.alfresco.bm.publicapi.factory.PublicApiFactory;
import org.alfresco.bm.site.SiteMember;

/**
 * Remove site member event processor.
 * 
 * @author steveglover
 *
 */
public class RemoveSiteMember extends AbstractRepositoryEventSelectorProcessor
{
    /**
     * @param services                access to collection services
     * @param publicApiFactory        a factory for creating connections to the public api
     * @param eventSelector           the event selector to use in selecting the next event
     */
    public RemoveSiteMember(Services services, PublicApiFactory publicApiFactory, EventSelector eventSelector)
    {
        super(services, publicApiFactory, eventSelector);
    }
    
    @Override
    public Object createDataObject(Object input, Object response) throws Exception
    {
    	RemoveSiteMemberRequest ret = null;

        if(input != null)
        {
	        Request request = (Request)input;
	        String networkId = request.getNetworkId();
	        String userId = request.getRunAsUserId();

        	SiteMember member = findSiteMember(request, response);
        	if(member == null)
        	{
        		member = randomSiteMember(userId, networkId, true);
        	}
        	
        	if(member != null)
        	{
        		ret = new RemoveSiteMemberRequest(networkId, userId, member.getSiteId(), member.getUsername());
        	}
        }
        
        return ret;
    }

    @Override
    protected EventProcessorResponse processEventImpl(Object input) throws Exception
    {
        EventProcessorResponse response = null;

        if(input == null)
        {
        	 response = new EventProcessorResponse("Skipping remove site member, input is null", true, null);
        }
        else
        {
            RemoveSiteMemberRequest removeSiteMemberRequest = (RemoveSiteMemberRequest)input;
            String userId = removeSiteMemberRequest.getRunAsUserId();
            String networkId = removeSiteMemberRequest.getNetworkId();
            String siteId = removeSiteMemberRequest.getSiteId();
            String memberId = removeSiteMemberRequest.getMemberId();

	        if(siteId != null && memberId != null && userId != null && networkId != null)
	        {
	        	// Note: don't record site removal in the siteMembers mirror.
	            getPublicApi(userId).deleteMember(networkId, siteId, memberId);
	            response = new EventProcessorResponse("Removed site member: " + removeSiteMemberRequest, true, null);
	        }
	        else
	        {
	            response = new EventProcessorResponse("Skipping remove site member: no siteId or memberId supplied.", true, null);
	        }
        }

        return response;
    }
}
