package org.alfresco.bm.publicapi.process;

import java.util.ArrayList;
import java.util.UUID;

import org.alfresco.bm.Services;
import org.alfresco.bm.event.selector.EventProcessorResponse;
import org.alfresco.bm.event.selector.EventSelector;
import org.alfresco.bm.publicapi.AbstractRepositoryEventSelectorProcessor;
import org.alfresco.bm.publicapi.data.Node;
import org.alfresco.bm.publicapi.data.Request;
import org.alfresco.bm.publicapi.factory.PublicApiFactory;
import org.springframework.social.alfresco.api.entities.AlfrescoList;
import org.springframework.social.alfresco.api.entities.Comment;

/**
 * Update comment event processor.
 * 
 * @author steveglover
 *
 */
public class UpdateComment extends AbstractRepositoryEventSelectorProcessor
{
    /**
     * @param services                access to collection services
     * @param publicApiFactory        a factory for creating connections to the public api
     * @param eventSelector           the event selector to use in selecting the next event
     */
    public UpdateComment(Services services, PublicApiFactory publicApiFactory, EventSelector eventSelector)
    {
        super(services, publicApiFactory, eventSelector);
    }

    protected String randomComment(Request request, String nodeId) throws Exception
    {
        String commentId = null;
        String networkId = request.getNetworkId();
        String userId = request.getRunAsUserId();
        
        int skipCount = 0;
        int maxItems = Integer.MAX_VALUE;
        Parameters parameters = new Parameters().addParameter("skipCount", String.valueOf(skipCount)).
                addParameter("maxItems", String.valueOf(maxItems));
        
        AlfrescoList<Comment> comments = getPublicApi(userId).getComments(networkId, nodeId, parameters.toMap());
        ArrayList<Comment> entries = comments.getEntries();
        int numComments = entries.size();
        if(numComments > 0)
        {
            int idx = random.nextInt(numComments);
            commentId = entries.get(idx).getId();
        }
        return commentId;
    }

    @Override
    public Object createDataObject(Object input, Object response) throws Exception
    {
        UpdateCommentRequest ret = null;

        if(input != null)
        {
            Request request = (Request)input;

	        Node node = getNode(request, response, matchAllNodes); // TODO need to check that the user can update the comment
	        if(node != null)
	        {
	            logger.debug("UpdateComment.createDataObject node = " + node);
	
	            String commentId = randomComment(request, node.getNodeId());
	            String content = UUID.randomUUID().toString();
	            String networkId = request.getNetworkId();
	            String userId = request.getRunAsUserId();
	    
	            ret = new UpdateCommentRequest(networkId, userId, node, commentId, content);
	        }
        }

        return ret;
    }

    @Override
    protected EventProcessorResponse processEventImpl(Object input) throws Exception
    {
        EventProcessorResponse response = null;

        if(input == null)
        {
        	 response = new EventProcessorResponse("Skipping update comment, input is null", true, null);
        }
        else
        {
            UpdateCommentRequest updateCommentRequest = (UpdateCommentRequest)input;
            String userId = updateCommentRequest.getRunAsUserId();
            String networkId = updateCommentRequest.getNetworkId();
            Node node = updateCommentRequest.getNode();
            String commentId = updateCommentRequest.getCommentId();
            String content = updateCommentRequest.getContent();

	        if(node != null && commentId != null && content != null)
	        {
	            getPublicApi(userId).updateComment(networkId, node.getNodeId(), commentId, content);
	            response = new EventProcessorResponse("Updated comment " + updateCommentRequest, true, node);
	        }
	        else
	        {
	            response = new EventProcessorResponse("Skipping update comment: no nodeId or commentId or content supplied.", true, null);
	        }
        }

        return response;
    }
}
