package org.alfresco.bm.publicapi.process;

import java.util.UUID;

import org.alfresco.bm.Services;
import org.alfresco.bm.event.selector.EventProcessorResponse;
import org.alfresco.bm.event.selector.EventSelector;
import org.alfresco.bm.publicapi.AbstractRepositoryEventSelectorProcessor;
import org.alfresco.bm.publicapi.data.AbstractDocumentContent;
import org.alfresco.bm.publicapi.data.DocumentNode;
import org.alfresco.bm.publicapi.data.Request;
import org.alfresco.bm.publicapi.data.StringDocumentContent;
import org.alfresco.bm.publicapi.factory.PublicApiFactory;
import org.apache.chemistry.opencmis.client.api.Document;
import org.apache.chemistry.opencmis.client.api.Session;

/**
 * Update content event processor.
 * 
 * @author steveglover
 *
 */
public class UpdateContent extends AbstractRepositoryEventSelectorProcessor
{
    /**
     * @param services                access to collection services
     * @param publicApiFactory        a factory for creating connections to the public api
     * @param eventSelector           the event selector to use in selecting the next event
     */
    public UpdateContent(Services services, PublicApiFactory publicApiFactory, EventSelector eventSelector)
    {
        super(services, publicApiFactory, eventSelector);
    }

    @Override
    public Object createDataObject(Object input, Object response) throws Exception
    {
        UpdateContentRequest ret = null;

        if(input != null)
        {
            Request request = (Request)input;

	        DocumentNode node = getDocument(request, response, canUpdateContentNodeMatcher);
	        if(node != null)
	        {
	            String networkId = request.getNetworkId();
	            String userId = request.getRunAsUserId();
	            AbstractDocumentContent content = new StringDocumentContent(UUID.randomUUID().toString(), UUID.randomUUID().toString());
	            ret = new UpdateContentRequest(networkId, userId, node, content);
	        }
        }
        
        return ret;
    }

    @Override
    protected EventProcessorResponse processEventImpl(Object input) throws Exception
    {
        EventProcessorResponse response = null;

        if(input == null)
        {
        	 response = new EventProcessorResponse("Unable to update content, input is null", true, null);
        }
        else
        {
            UpdateContentRequest updateContentRequest = (UpdateContentRequest)input;
            String userId = updateContentRequest.getRunAsUserId();
            String networkId = updateContentRequest.getNetworkId();
            DocumentNode node = updateContentRequest.getNode();
            AbstractDocumentContent content = updateContentRequest.getContent();
            
            if(node == null)
            {
            	response = new EventProcessorResponse("Failed to update content, nodeId is null", false, null);
	        }
	        else if(userId == null)
	        {
	            response = new EventProcessorResponse("Failed to update content, userId is null", false, null);
	        }
	        else if(networkId == null)
	        {
	            response = new EventProcessorResponse("Failed to update content, networkId is null", false, null);
	        }
	        else if(content == null)
	        {
	            response = new EventProcessorResponse("Failed to update content, content is null", false, null);
	        }
	        else
	        {
	            Session cmisSession = getPublicApi(userId).getCMISSession(networkId);
	            Document doc = (Document)cmisSession.getObject(node.getNodeId());
	            doc.setContentStream(content.getContentStream(), true);
	            node.setContent(content);
	
	            response = new EventProcessorResponse("Updated content '" + node, true, node, true);
	        }
        }

        return response;
    }
}
