package org.alfresco.bm.publicapi.process;

import java.util.Map;

import org.alfresco.bm.Services;
import org.alfresco.bm.event.selector.EventProcessorResponse;
import org.alfresco.bm.event.selector.EventSelector;
import org.alfresco.bm.publicapi.AbstractRepositoryEventSelectorProcessor;
import org.alfresco.bm.publicapi.data.Node;
import org.alfresco.bm.publicapi.data.Request;
import org.alfresco.bm.publicapi.factory.PublicApiFactory;
import org.apache.chemistry.opencmis.client.api.CmisObject;
import org.apache.chemistry.opencmis.client.api.Session;

/**
 * Update object properties.
 * 
 * @author steveglover
 *
 */
public class UpdateProperties extends AbstractRepositoryEventSelectorProcessor
{
    /**
     * @param services                access to collection services
     * @param publicApiFactory        a factory for creating connections to the public api
     * @param eventSelector           the event selector to use in selecting the next event
     */
    public UpdateProperties(Services services, PublicApiFactory publicApiFactory, EventSelector eventSelector)
    {
        super(services, publicApiFactory, eventSelector);
    }

    @Override
    public Object createDataObject(Object input, Object response) throws Exception
    {
        UpdatePropertiesRequest ret = null;

        if(input != null)
        {
            Request request = (Request)input;
	        Node node = getNode(request, response, canUpdatePropertiesNodeMatcher);
	        if(node != null)
	        {
	            logger.debug("UpdateProperties.createDataObject Object = " + node);
	
	            String networkId = request.getNetworkId();
	            String userId = request.getRunAsUserId();
	            Map<String, Object> properties = getProperties();
	            ret = new UpdatePropertiesRequest(networkId, userId, node, properties);
	        }
        }
        
        return ret;
    }

    @Override
    protected EventProcessorResponse processEventImpl(Object input) throws Exception
    {
        EventProcessorResponse response = null;

        if(input == null)
        {
            response = new EventProcessorResponse("Skipping update properties, null input" , true, null);
        }
        else
        {
	    	UpdatePropertiesRequest updatePropertiesRequest = (UpdatePropertiesRequest)input;
	        String runAsUserId = updatePropertiesRequest.getRunAsUserId();
	        String networkId = updatePropertiesRequest.getNetworkId();
	        Node node = updatePropertiesRequest.getNode();
	        Map<String, Object> properties = updatePropertiesRequest.getProperties();
	
	        if(node == null)
	        {
	            response = new EventProcessorResponse("Skipping update properties, node is null" , true, null);
	        }
	        else if(properties == null)
	        {
	            response = new EventProcessorResponse("Skipping update properties, properties is null" , true, null);
	        }
	        else if(networkId == null)
	        {
	            response = new EventProcessorResponse("Skipping update properties, networkId is null" , true, null);
	        }
	        else if(runAsUserId == null)
	        {
	            response = new EventProcessorResponse("Skipping update properties, runAsUserId is null" , true, null);
	        }
	        else
	        {
	        	Session cmisSession = getPublicApi(runAsUserId).getCMISSession(networkId);
	        	CmisObject obj = cmisSession.getObject(node.getNodeId());
	        	if(obj != null)
	        	{
	        		obj.updateProperties(properties, true);
	        		Node updatedNode = Node.createNode(obj);
	                response = new EventProcessorResponse("Updated properties " + updatedNode, true, updatedNode, true);
	        	}
	        	else
	        	{
	                response = new EventProcessorResponse("Failed to update properties, cannot find object for id " + node.getNodeId(), true, null);        		
	        	}
	        }
        }

        return response;
    }
}
