package org.alfresco.bm.publicapi.process;

import java.util.ArrayList;

import org.alfresco.bm.Services;
import org.alfresco.bm.event.selector.EventProcessorResponse;
import org.alfresco.bm.event.selector.EventSelector;
import org.alfresco.bm.publicapi.AbstractRepositoryEventSelectorProcessor;
import org.alfresco.bm.publicapi.data.Request;
import org.alfresco.bm.publicapi.factory.PublicApiFactory;
import org.springframework.social.alfresco.api.entities.AlfrescoList;
import org.springframework.social.alfresco.api.entities.Member;
import org.springframework.social.alfresco.api.entities.Role;

/**
 * Update site membership event processor.
 * 
 * @author steveglover
 *
 */
public class UpdateSiteMember extends AbstractRepositoryEventSelectorProcessor
{
    /**
     * @param services                access to collection services
     * @param publicApiFactory        a factory for creating connections to the public api
     * @param eventSelector           the event selector to use in selecting the next event
     */
    public UpdateSiteMember(Services services, PublicApiFactory publicApiFactory, EventSelector eventSelector)
    {
        super(services, publicApiFactory, eventSelector);
    }

    protected Role randomRole(Role notThisRole)
    {
        Role role = null;
        Role[] roles = Role.values();
        int idx = -1;
        do
        {
            idx = random.nextInt(roles.length);
            role = roles[idx];
        } while(role.equals(notThisRole));

        return role;
    }

    protected Member randomMember(Request request, String networkId, String siteId) throws Exception
    {
        Member member = null;
        String userId = request.getRunAsUserId();
        
        int skipCount = 0;
        int maxItems = Integer.MAX_VALUE;
        Parameters parameters = new Parameters().addParameter("skipCount", String.valueOf(skipCount)).
                addParameter("maxItems", String.valueOf(maxItems));

        AlfrescoList<Member> members = getPublicApi(userId).getMembers(networkId, siteId, parameters.toMap());
        ArrayList<Member> entries = members.getEntries();
        int numMembers = entries.size();
        if(numMembers > 0)
        {
            int idx = random.nextInt(numMembers);
            member = entries.get(idx);
        }
        return member;
    }

    @Override
    public Object createDataObject(Object input, Object response) throws Exception
    {
    	UpdateSiteMemberRequest ret = null;

        if(input != null)
        {
	        Request request = (Request)input;
	        String networkId = request.getNetworkId();
	        String userId = request.getRunAsUserId();
	        
	        String siteId = randomSite(networkId);
	        Member member = randomMember(request, networkId, siteId);
	        if(member != null)
	        {
	            Role role = member.getRole();
	            member.setRole(randomRole(role));
	
	            ret = new UpdateSiteMemberRequest(networkId, userId, siteId, member.getId(), member);
	            return ret;
	        }
	        else
	        {
	            throw new RuntimeException("Unable to determine site member");
	        }
        }
        
        return ret;
    }

    @Override
    protected EventProcessorResponse processEventImpl(Object input) throws Exception
    {
        EventProcessorResponse response = null;

        if(input == null)
        {
            response = new EventProcessorResponse("Skipping update site member, null input" , true, null);
        }
        else
        {
            UpdateSiteMemberRequest updateSiteMemberRequest = (UpdateSiteMemberRequest)input;
            String userId = updateSiteMemberRequest.getRunAsUserId();
            String networkId = updateSiteMemberRequest.getNetworkId();
            String siteId = updateSiteMemberRequest.getSiteId();
            String memberId = updateSiteMemberRequest.getMemberId();
            Member member = updateSiteMemberRequest.getMember();

	        if(siteId != null && memberId != null && member != null && userId != null && networkId != null)
	        {
	            getPublicApi(userId).updateMember(networkId, siteId, memberId, member.getRole());
	            response = new EventProcessorResponse("Updated site member: " + updateSiteMemberRequest, true, null);
	        }
	        else
	        {
	            response = new EventProcessorResponse("Skipping update comment: no siteId or memberId or member supplied.", true, null);
	        }
        }

        return response;
    }
}
